@@ -339,7 +339,7 @@ static HRESULT CLRRuntimeInfo_GetRuntimeHost(CLRRuntimeInfo *This, RuntimeHost *
         return hr;
     }
 
-    if (!get_mono_path(mono_path))
+    if (!get_mono_path(mono_path, FALSE))
     {
         ERR("Wine Mono is not installed\n");
         return CLR_E_SHIM_RUNTIME;
@@ -778,9 +778,9 @@ static BOOL get_mono_path_datadir(LPWSTR path)
     return ret;
 }
 
-BOOL get_mono_path(LPWSTR path)
+BOOL get_mono_path(LPWSTR path, BOOL skip_local)
 {
-    return get_mono_path_local(path) ||
+    return (!skip_local && get_mono_path_local(path)) ||
         get_mono_path_registry(path) ||
         get_mono_path_datadir(path) ||
         get_mono_path_unix(INSTALL_DATADIR "/wine/mono/wine-mono-" WINE_MONO_VERSION, path) ||
@@ -1723,11 +1723,35 @@ HRESULT get_runtime_info(LPCWSTR exefile, LPCWSTR version, LPCWSTR config_file,
 
     if (exefile && !version)
     {
+        DWORD major, minor, build;
+
         hr = CLRMetaHost_GetVersionFromFile(0, exefile, local_version, &local_version_size);
 
         version = local_version;
 
         if (FAILED(hr)) return hr;
+
+        /* When running an executable, specifically when getting the version number from
+         * the exe, native accepts a matching major.minor with build <= expected build. */
+        if (!parse_runtime_version(version, &major, &minor, &build))
+        {
+            ERR("Cannot parse %s\n", debugstr_w(version));
+            return CLR_E_SHIM_RUNTIME;
+        }
+
+        if (legacy)
+            i = 3;
+        else
+            i = NUM_RUNTIMES;
+
+        while (i--)
+        {
+            if (runtimes[i].major == major && runtimes[i].minor == minor && runtimes[i].build >= build)
+            {
+                return ICLRRuntimeInfo_QueryInterface(&runtimes[i].ICLRRuntimeInfo_iface,
+                        &IID_ICLRRuntimeInfo, (void **)result);
+            }
+        }
     }
 
     if (version)
