@@ -542,8 +542,7 @@ static inline unsigned char
 ulaw2alaw(unsigned char uval)
 {
     uval &= 0xff;
-    return (unsigned char) ((uval & 0x80) ? (0xD5 ^ (_u2a[0xFF ^ uval] - 1)) :
-                            (unsigned char) (0x55 ^ (_u2a[0x7F ^ uval] - 1)));
+    return (uval & 0x80) ? (0xD5 ^ (_u2a[0xFF ^ uval] - 1)) : (0x55 ^ (_u2a[0x7F ^ uval] - 1));
 }
 
 /* -------------------------------------------------------------------------------*/
