@@ -80,7 +80,7 @@ static HRESULT return_nscstr(nsresult nsres, nsACString *nscstr, BSTR *p)
 
     if(*str) {
         len = MultiByteToWideChar(CP_UTF8, 0, str, -1, NULL, 0);
-        *p = SysAllocStringLen(NULL, len);
+        *p = SysAllocStringLen(NULL, len - 1);
         if(!*p) {
             nsACString_Finish(nscstr);
             return E_OUTOFMEMORY;
