@@ -169,7 +169,7 @@ static void release_typelib(void)
     if(!typelib)
         return;
 
-    for(i=0; i < sizeof(typeinfos)/sizeof(*typeinfos); i++)
+    for(i = 0; i < ARRAY_SIZE(typeinfos); i++)
         if(typeinfos[i])
             ITypeInfo_Release(typeinfos[i]);
 
@@ -792,8 +792,10 @@ static HRESULT WINAPI ScriptControl_put_State(IScriptControl *iface, ScriptContr
 static HRESULT WINAPI ScriptControl_put_SitehWnd(IScriptControl *iface, LONG hwnd)
 {
     ScriptControl *This = impl_from_IScriptControl(iface);
+
     FIXME("(%p)->(%x)\n", This, hwnd);
-    return E_NOTIMPL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptControl_get_SitehWnd(IScriptControl *iface, LONG *p)
@@ -973,8 +975,27 @@ static HRESULT WINAPI ScriptControl_AddCode(IScriptControl *iface, BSTR code)
 static HRESULT WINAPI ScriptControl_Eval(IScriptControl *iface, BSTR expression, VARIANT *res)
 {
     ScriptControl *This = impl_from_IScriptControl(iface);
+    EXCEPINFO excepinfo;
+    HRESULT hr;
+
     FIXME("(%p)->(%s %p)\n", This, debugstr_w(expression), res);
-    return E_NOTIMPL;
+
+    if (!res)
+        return E_POINTER;
+    V_VT(res) = VT_EMPTY;
+
+    if (!This->host || This->state != Initialized)
+        return E_FAIL;
+
+    hr = IActiveScript_SetScriptState(This->host->script, SCRIPTSTATE_STARTED);
+    if (FAILED(hr))
+        return hr;
+
+    hr = IActiveScriptParse_ParseScriptText(This->host->parse, expression, NULL, NULL, NULL,
+                                            0, 1, SCRIPTTEXT_ISEXPRESSION, res, &excepinfo);
+    /* FIXME: more error hanlding */
+
+    return hr;
 }
 
 static HRESULT WINAPI ScriptControl_ExecuteStatement(IScriptControl *iface, BSTR statement)
