@@ -29,7 +29,9 @@
 #include "winternl.h"
 #include "ddk/ntddk.h"
 #include "ddk/wdm.h"
+#include "ddk/ntifs.h"
 
+#include "wine/asm.h"
 #include "wine/debug.h"
 #include "wine/heap.h"
 
@@ -47,14 +49,7 @@ enum object_type
     TYPE_AUTO_TIMER = 9,
 };
 
-static CRITICAL_SECTION sync_cs;
-static CRITICAL_SECTION_DEBUG sync_cs_debug =
-{
-    0, 0, &sync_cs,
-    { &sync_cs_debug.ProcessLocksList, &sync_cs_debug.ProcessLocksList },
-    0, 0, { (DWORD_PTR)(__FILE__ ": sync_cs") }
-};
-static CRITICAL_SECTION sync_cs = { &sync_cs_debug, -1, 0, 0, 0, 0 };
+DECLARE_CRITICAL_SECTION(sync_cs);
 
 /***********************************************************************
  *           KeWaitForMultipleObjects   (NTOSKRNL.EXE.@)
@@ -80,7 +75,7 @@ NTSTATUS WINAPI KeWaitForMultipleObjects(ULONG count, void *pobjs[],
     {
         if (objs[i]->WaitListHead.Blink == INVALID_HANDLE_VALUE)
         {
-            handles[i] = kernel_object_handle( objs[i], SYNCHRONIZE );
+            ObOpenObjectByPointer( objs[i], OBJ_KERNEL_HANDLE, NULL, SYNCHRONIZE, NULL, KernelMode, &handles[i] );
             continue;
         }
 
@@ -266,7 +261,7 @@ LONG WINAPI KeSetEvent( PRKEVENT event, KPRIORITY increment, BOOLEAN wait )
     }
     else
     {
-        if ((handle = kernel_object_handle( event, EVENT_MODIFY_STATE )))
+        if (!ObOpenObjectByPointer( event, OBJ_KERNEL_HANDLE, NULL, EVENT_MODIFY_STATE, NULL, KernelMode, &handle ))
         {
             NtSetEvent( handle, &ret );
             NtClose( handle );
@@ -297,7 +292,7 @@ LONG WINAPI KeResetEvent( PRKEVENT event )
     }
     else
     {
-        if ((handle = kernel_object_handle( event, EVENT_MODIFY_STATE )))
+        if (!ObOpenObjectByPointer( event, OBJ_KERNEL_HANDLE, NULL, EVENT_MODIFY_STATE, NULL, KernelMode, &handle ))
         {
             NtResetEvent( handle, &ret );
             NtClose( handle );
@@ -513,7 +508,7 @@ void WINAPI KeReleaseSpinLockFromDpcLevel( KSPIN_LOCK *lock )
  *           KeAcquireInStackQueuedSpinLockAtDpcLevel  (NTOSKRNL.EXE.@)
  */
 DEFINE_FASTCALL_WRAPPER( KeAcquireInStackQueuedSpinLockAtDpcLevel, 8 )
-void WINAPI KeAcquireInStackQueuedSpinLockAtDpcLevel( KSPIN_LOCK *lock, KLOCK_QUEUE_HANDLE *queue )
+void FASTCALL KeAcquireInStackQueuedSpinLockAtDpcLevel( KSPIN_LOCK *lock, KLOCK_QUEUE_HANDLE *queue )
 {
     KSPIN_LOCK_QUEUE *tail;
 
@@ -540,7 +535,7 @@ void WINAPI KeAcquireInStackQueuedSpinLockAtDpcLevel( KSPIN_LOCK *lock, KLOCK_QU
  *           KeReleaseInStackQueuedSpinLockFromDpcLevel  (NTOSKRNL.EXE.@)
  */
 DEFINE_FASTCALL1_WRAPPER( KeReleaseInStackQueuedSpinLockFromDpcLevel )
-void WINAPI KeReleaseInStackQueuedSpinLockFromDpcLevel( KLOCK_QUEUE_HANDLE *queue )
+void FASTCALL KeReleaseInStackQueuedSpinLockFromDpcLevel( KLOCK_QUEUE_HANDLE *queue )
 {
     KSPIN_LOCK *lock = (KSPIN_LOCK *)((ULONG_PTR)queue->LockQueue.Lock & ~QUEUED_SPINLOCK_OWNED);
     KSPIN_LOCK_QUEUE *next;
@@ -627,7 +622,7 @@ void WINAPI IoReleaseCancelSpinLock( KIRQL irql )
  *           ExfInterlockedRemoveHeadList  (NTOSKRNL.EXE.@)
  */
 DEFINE_FASTCALL_WRAPPER( ExfInterlockedRemoveHeadList, 8 )
-PLIST_ENTRY WINAPI ExfInterlockedRemoveHeadList( LIST_ENTRY *list, KSPIN_LOCK *lock )
+PLIST_ENTRY FASTCALL ExfInterlockedRemoveHeadList( LIST_ENTRY *list, KSPIN_LOCK *lock )
 {
     return ExInterlockedRemoveHeadList( list, lock );
 }
@@ -654,7 +649,7 @@ LIST_ENTRY * WINAPI ExInterlockedRemoveHeadList( LIST_ENTRY *list, KSPIN_LOCK *l
  *           InterlockedPopEntrySList   (NTOSKRNL.EXE.@)
  */
 DEFINE_FASTCALL1_WRAPPER( NTOSKRNL_InterlockedPopEntrySList )
-PSLIST_ENTRY WINAPI NTOSKRNL_InterlockedPopEntrySList( PSLIST_HEADER list )
+PSLIST_ENTRY FASTCALL NTOSKRNL_InterlockedPopEntrySList( PSLIST_HEADER list )
 {
     return RtlInterlockedPopEntrySList( list );
 }
@@ -664,7 +659,7 @@ PSLIST_ENTRY WINAPI NTOSKRNL_InterlockedPopEntrySList( PSLIST_HEADER list )
  *           InterlockedPushEntrySList   (NTOSKRNL.EXE.@)
  */
 DEFINE_FASTCALL_WRAPPER( NTOSKRNL_InterlockedPushEntrySList, 8 )
-PSLIST_ENTRY WINAPI NTOSKRNL_InterlockedPushEntrySList( PSLIST_HEADER list, PSLIST_ENTRY entry )
+PSLIST_ENTRY FASTCALL NTOSKRNL_InterlockedPushEntrySList( PSLIST_HEADER list, PSLIST_ENTRY entry )
 {
     return RtlInterlockedPushEntrySList( list, entry );
 }
@@ -674,7 +669,7 @@ PSLIST_ENTRY WINAPI NTOSKRNL_InterlockedPushEntrySList( PSLIST_HEADER list, PSLI
  *           ExInterlockedPopEntrySList   (NTOSKRNL.EXE.@)
  */
 DEFINE_FASTCALL_WRAPPER( NTOSKRNL_ExInterlockedPopEntrySList, 8 )
-PSLIST_ENTRY WINAPI NTOSKRNL_ExInterlockedPopEntrySList( PSLIST_HEADER list, PKSPIN_LOCK lock )
+PSLIST_ENTRY FASTCALL NTOSKRNL_ExInterlockedPopEntrySList( PSLIST_HEADER list, PKSPIN_LOCK lock )
 {
     return RtlInterlockedPopEntrySList( list );
 }
@@ -684,17 +679,27 @@ PSLIST_ENTRY WINAPI NTOSKRNL_ExInterlockedPopEntrySList( PSLIST_HEADER list, PKS
  *           ExInterlockedPushEntrySList   (NTOSKRNL.EXE.@)
  */
 DEFINE_FASTCALL_WRAPPER( NTOSKRNL_ExInterlockedPushEntrySList, 12 )
-PSLIST_ENTRY WINAPI NTOSKRNL_ExInterlockedPushEntrySList( PSLIST_HEADER list, PSLIST_ENTRY entry, PKSPIN_LOCK lock )
+PSLIST_ENTRY FASTCALL NTOSKRNL_ExInterlockedPushEntrySList( PSLIST_HEADER list, PSLIST_ENTRY entry, PKSPIN_LOCK lock )
 {
     return RtlInterlockedPushEntrySList( list, entry );
 }
 
 
+/***********************************************************************
+ *           ExInterlockedFlushSList   (NTOSKRNL.EXE.@)
+ */
+DEFINE_FASTCALL1_WRAPPER( NTOSKRNL_ExInterlockedFlushSList )
+PSLIST_ENTRY FASTCALL NTOSKRNL_ExInterlockedFlushSList( PSLIST_HEADER list )
+{
+    return RtlInterlockedFlushSList( list );
+}
+
+
 /***********************************************************************
  *           ExAcquireFastMutexUnsafe  (NTOSKRNL.EXE.@)
  */
 DEFINE_FASTCALL1_WRAPPER(ExAcquireFastMutexUnsafe)
-void WINAPI ExAcquireFastMutexUnsafe( FAST_MUTEX *mutex )
+void FASTCALL ExAcquireFastMutexUnsafe( FAST_MUTEX *mutex )
 {
     LONG count;
 
@@ -709,7 +714,7 @@ void WINAPI ExAcquireFastMutexUnsafe( FAST_MUTEX *mutex )
  *           ExReleaseFastMutexUnsafe  (NTOSKRNL.EXE.@)
  */
 DEFINE_FASTCALL1_WRAPPER(ExReleaseFastMutexUnsafe)
-void WINAPI ExReleaseFastMutexUnsafe( FAST_MUTEX *mutex )
+void FASTCALL ExReleaseFastMutexUnsafe( FAST_MUTEX *mutex )
 {
     LONG count;
 
@@ -720,6 +725,28 @@ void WINAPI ExReleaseFastMutexUnsafe( FAST_MUTEX *mutex )
         KeSetEvent( &mutex->Event, IO_NO_INCREMENT, FALSE );
 }
 
+#ifndef __i386__
+
+/***********************************************************************
+ *           ExAcquireFastMutex    (NTOSKRNL.@)
+ */
+void WINAPI ExAcquireFastMutex( FAST_MUTEX *mutex )
+{
+    /* FIXME: lower IRQL */
+    ExAcquireFastMutexUnsafe( mutex );
+}
+
+/***********************************************************************
+ *           ExReleaseFastMutex    (NTOSKRNL.@)
+ */
+void WINAPI ExReleaseFastMutex( FAST_MUTEX *mutex )
+{
+    ExReleaseFastMutexUnsafe( mutex );
+    /* FIXME: restore IRQL */
+}
+
+#endif /* __i386__ */
+
 /* Use of the fields of an ERESOURCE structure seems to vary wildly between
  * Windows versions. The below implementation uses them as follows:
  *
@@ -1083,7 +1110,7 @@ void WINAPI ExReleaseResourceForThreadLite( ERESOURCE *resource, ERESOURCE_THREA
  *           ExReleaseResourceLite  (NTOSKRNL.EXE.@)
  */
 DEFINE_FASTCALL1_WRAPPER( ExReleaseResourceLite )
-void WINAPI ExReleaseResourceLite( ERESOURCE *resource )
+void FASTCALL ExReleaseResourceLite( ERESOURCE *resource )
 {
     ExReleaseResourceForThreadLite( resource, (ERESOURCE_THREAD)KeGetCurrentThread() );
 }
