@@ -29,15 +29,21 @@
 #include "winternl.h"
 #include "winioctl.h"
 #include "ddk/ntddk.h"
+#include "ddk/ntifs.h"
 #include "ddk/wdm.h"
 
 #include "driver.h"
 
-static const WCHAR driver_device[] = {'\\','D','e','v','i','c','e',
-                                      '\\','W','i','n','e','T','e','s','t','D','r','i','v','e','r',0};
+static const WCHAR device_name[] = {'\\','D','e','v','i','c','e',
+                                    '\\','W','i','n','e','T','e','s','t','D','r','i','v','e','r',0};
+static const WCHAR upper_name[] = {'\\','D','e','v','i','c','e',
+                                   '\\','W','i','n','e','T','e','s','t','U','p','p','e','r',0};
 static const WCHAR driver_link[] = {'\\','D','o','s','D','e','v','i','c','e','s',
                                     '\\','W','i','n','e','T','e','s','t','D','r','i','v','e','r',0};
 
+static DRIVER_OBJECT *driver_obj;
+static DEVICE_OBJECT *lower_device, *upper_device;
+
 static HANDLE okfile;
 static LONG successes;
 static LONG failures;
@@ -50,9 +56,13 @@ static int winetest_debug;
 static int winetest_report_success;
 
 static POBJECT_TYPE *pExEventObjectType, *pIoFileObjectType, *pPsThreadType;
+static PEPROCESS *pPsInitialSystemProcess;
+static void *create_caller_thread;
 
 void WINAPI ObfReferenceObject( void *obj );
 
+NTSTATUS WINAPI ZwQueryInformationProcess(HANDLE,PROCESSINFOCLASS,void*,ULONG,ULONG*);
+
 extern int CDECL _vsnprintf(char *str, size_t len, const char *format, __ms_va_list argptr);
 
 static void kvprintf(const char *format, __ms_va_list ap)
@@ -169,6 +179,11 @@ static void winetest_end_todo(void)
     todo_level >>= 1;
 }
 
+static int broken(int condition)
+{
+    return !running_under_wine && condition;
+}
+
 #define ok(condition, ...)  ok_(__FILE__, __LINE__, condition, __VA_ARGS__)
 #define todo_if(is_todo) for (winetest_start_todo(is_todo); \
                               winetest_loop_todo(); \
@@ -222,10 +237,14 @@ static void test_irp_struct(IRP *irp, DEVICE_OBJECT *device)
 {
     IO_STACK_LOCATION *irpsp = IoGetCurrentIrpStackLocation( irp );
 
+    ok(device == upper_device, "Expected device %p, got %p.\n", upper_device, device);
     ok(last_created_file != NULL, "last_created_file = NULL\n");
     ok(irpsp->FileObject == last_created_file, "FileObject != last_created_file\n");
-    ok(irpsp->DeviceObject == device, "unexpected DeviceObject\n");
-    ok(irpsp->FileObject->DeviceObject == device, "unexpected FileObject->DeviceObject\n");
+    ok(irpsp->DeviceObject == upper_device, "unexpected DeviceObject\n");
+    ok(irpsp->FileObject->DeviceObject == lower_device, "unexpected FileObject->DeviceObject\n");
+    ok(!irp->UserEvent, "UserEvent = %p\n", irp->UserEvent);
+    ok(irp->Tail.Overlay.Thread == (PETHREAD)KeGetCurrentThread(),
+       "IRP thread is not the current thread\n");
 }
 
 static void test_mdl_map(void)
@@ -243,7 +262,7 @@ static void test_mdl_map(void)
 todo_wine
     ok(addr != NULL, "MmMapLockedPagesSpecifyCache failed\n");
 
-    /* MmUnmapLockedPages(addr, mdl); */
+    MmUnmapLockedPages(addr, mdl);
 
     IoFreeMdl(mdl);
 }
@@ -313,19 +332,71 @@ static NTSTATUS wait_single_handle(HANDLE handle, ULONGLONG timeout)
     return ZwWaitForSingleObject(handle, FALSE, &integer);
 }
 
-static void test_currentprocess(void)
+static void test_current_thread(BOOL is_system)
 {
+    PROCESS_BASIC_INFORMATION info;
+    DISPATCHER_HEADER *header;
+    HANDLE process_handle, id;
     PEPROCESS current;
     PETHREAD thread;
     NTSTATUS ret;
 
     current = IoGetCurrentProcess();
-todo_wine
     ok(current != NULL, "Expected current process to be non-NULL\n");
 
+    header = (DISPATCHER_HEADER*)current;
+    ok(header->Type == 3, "header->Type != 3, = %u\n", header->Type);
+    ret = wait_single(current, 0);
+    ok(ret == STATUS_TIMEOUT, "got %#x\n", ret);
+
+    if (is_system)
+        ok(current == *pPsInitialSystemProcess, "current != PsInitialSystemProcess\n");
+    else
+        ok(current != *pPsInitialSystemProcess, "current == PsInitialSystemProcess\n");
+
+    ok(PsGetProcessId(current) == PsGetCurrentProcessId(), "process IDs don't match\n");
+    ok(PsGetThreadProcessId((PETHREAD)KeGetCurrentThread()) == PsGetCurrentProcessId(), "process IDs don't match\n");
+
     thread = PsGetCurrentThread();
     ret = wait_single( thread, 0 );
     ok(ret == STATUS_TIMEOUT, "got %#x\n", ret);
+
+    ok(PsGetThreadId((PETHREAD)KeGetCurrentThread()) == PsGetCurrentThreadId(), "thread IDs don't match\n");
+    ok(PsIsSystemThread((PETHREAD)KeGetCurrentThread()) == is_system, "unexpected system thread\n");
+    if (!is_system)
+        ok(create_caller_thread == KeGetCurrentThread(), "thread is not create caller thread\n");
+
+    ret = ObOpenObjectByPointer(current, OBJ_KERNEL_HANDLE, NULL, PROCESS_QUERY_INFORMATION, NULL, KernelMode, &process_handle);
+    ok(!ret, "ObOpenObjectByPointer failed: %#x\n", ret);
+
+    ret = ZwQueryInformationProcess(process_handle, ProcessBasicInformation, &info, sizeof(info), NULL);
+    ok(!ret, "ZwQueryInformationProcess failed: %#x\n", ret);
+
+    id = PsGetProcessInheritedFromUniqueProcessId(current);
+    ok(id == (HANDLE)info.InheritedFromUniqueProcessId, "unexpected process id %p\n", id);
+
+    ret = ZwClose(process_handle);
+    ok(!ret, "ZwClose failed: %#x\n", ret);
+}
+
+static void test_critical_region(BOOL is_dispatcher)
+{
+    BOOLEAN result;
+
+    KeEnterCriticalRegion();
+    KeEnterCriticalRegion();
+
+    result = KeAreApcsDisabled();
+    ok(result == TRUE, "KeAreApcsDisabled returned %x\n", result);
+    KeLeaveCriticalRegion();
+
+    result = KeAreApcsDisabled();
+    ok(result == TRUE, "KeAreApcsDisabled returned %x\n", result);
+    KeLeaveCriticalRegion();
+
+    result = KeAreApcsDisabled();
+    ok(result == is_dispatcher || broken(is_dispatcher && !result),
+       "KeAreApcsDisabled returned %x\n", result);
 }
 
 static void sleep(void)
@@ -625,13 +696,13 @@ static void test_sync(void)
     /* test timers */
     KeInitializeTimerEx(&timer, NotificationTimer);
 
-    timeout.QuadPart = -100;
+    timeout.QuadPart = -20 * 10000;
     KeSetTimerEx(&timer, timeout, 0, NULL);
 
     ret = wait_single(&timer, 0);
     ok(ret == WAIT_TIMEOUT, "got %#x\n", ret);
 
-    ret = wait_single(&timer, -200);
+    ret = wait_single(&timer, -40 * 10000);
     ok(ret == 0, "got %#x\n", ret);
 
     ret = wait_single(&timer, 0);
@@ -645,33 +716,225 @@ static void test_sync(void)
     ret = wait_single(&timer, 0);
     ok(ret == WAIT_TIMEOUT, "got %#x\n", ret);
 
-    ret = wait_single(&timer, -200);
+    ret = wait_single(&timer, -40 * 10000);
     ok(ret == 0, "got %#x\n", ret);
 
-    ret = wait_single(&timer, 0);
+    ret = wait_single(&timer, -40 * 10000);
     ok(ret == WAIT_TIMEOUT, "got %#x\n", ret);
 
     KeCancelTimer(&timer);
-    KeSetTimerEx(&timer, timeout, 10, NULL);
+    KeSetTimerEx(&timer, timeout, 20, NULL);
 
     ret = wait_single(&timer, 0);
     ok(ret == WAIT_TIMEOUT, "got %#x\n", ret);
 
-    ret = wait_single(&timer, -200);
+    ret = wait_single(&timer, -40 * 10000);
     ok(ret == 0, "got %#x\n", ret);
 
     ret = wait_single(&timer, 0);
     ok(ret == WAIT_TIMEOUT, "got %#x\n", ret);
 
-    ret = wait_single(&timer, -20 * 10000);
+    ret = wait_single(&timer, -40 * 10000);
     ok(ret == 0, "got %#x\n", ret);
 
-    ret = wait_single(&timer, -20 * 10000);
+    ret = wait_single(&timer, -40 * 10000);
     ok(ret == 0, "got %#x\n", ret);
 
     KeCancelTimer(&timer);
 }
 
+static void test_call_driver(DEVICE_OBJECT *device)
+{
+    IO_STACK_LOCATION *irpsp;
+    IO_STATUS_BLOCK iosb;
+    IRP *irp = NULL;
+    KEVENT event;
+    NTSTATUS status;
+
+    irp = IoBuildAsynchronousFsdRequest(IRP_MJ_FLUSH_BUFFERS, device, NULL, 0, NULL, &iosb);
+    ok(irp->UserIosb == &iosb, "unexpected UserIosb\n");
+    ok(!irp->Cancel, "Cancel = %x\n", irp->Cancel);
+    ok(!irp->CancelRoutine, "CancelRoutine = %x\n", irp->CancelRoutine);
+    ok(!irp->UserEvent, "UserEvent = %p\n", irp->UserEvent);
+    ok(irp->CurrentLocation == 2, "CurrentLocation = %u\n", irp->CurrentLocation);
+    ok(irp->Tail.Overlay.Thread == (PETHREAD)KeGetCurrentThread(),
+       "IRP thread is not the current thread\n");
+
+    irpsp = IoGetNextIrpStackLocation(irp);
+    ok(irpsp->MajorFunction == IRP_MJ_FLUSH_BUFFERS, "MajorFunction = %u\n", irpsp->MajorFunction);
+    ok(!irpsp->DeviceObject, "DeviceObject = %u\n", irpsp->DeviceObject);
+    ok(!irpsp->FileObject, "FileObject = %u\n", irpsp->FileObject);
+    ok(!irpsp->CompletionRoutine, "CompletionRouptine = %p\n", irpsp->CompletionRoutine);
+
+    status = IoCallDriver(device, irp);
+    ok(status == STATUS_PENDING, "IoCallDriver returned %#x\n", status);
+
+    irp->IoStatus.Status = STATUS_SUCCESS;
+    irp->IoStatus.Information = 0;
+    IoCompleteRequest(irp, IO_NO_INCREMENT);
+
+    KeInitializeEvent(&event, NotificationEvent, FALSE);
+
+    irp = IoBuildSynchronousFsdRequest(IRP_MJ_FLUSH_BUFFERS, device, NULL, 0, NULL, &event, &iosb);
+    ok(irp->UserIosb == &iosb, "unexpected UserIosb\n");
+    ok(!irp->Cancel, "Cancel = %x\n", irp->Cancel);
+    ok(!irp->CancelRoutine, "CancelRoutine = %x\n", irp->CancelRoutine);
+    ok(irp->UserEvent == &event, "UserEvent = %p\n", irp->UserEvent);
+    ok(irp->CurrentLocation == 2, "CurrentLocation = %u\n", irp->CurrentLocation);
+    ok(irp->Tail.Overlay.Thread == (PETHREAD)KeGetCurrentThread(),
+       "IRP thread is not the current thread\n");
+
+    irpsp = IoGetNextIrpStackLocation(irp);
+    ok(irpsp->MajorFunction == IRP_MJ_FLUSH_BUFFERS, "MajorFunction = %u\n", irpsp->MajorFunction);
+    ok(!irpsp->DeviceObject, "DeviceObject = %u\n", irpsp->DeviceObject);
+    ok(!irpsp->FileObject, "FileObject = %u\n", irpsp->FileObject);
+    ok(!irpsp->CompletionRoutine, "CompletionRouptine = %p\n", irpsp->CompletionRoutine);
+
+    status = wait_single(&event, 0);
+    ok(status == STATUS_TIMEOUT, "got %#x\n", status);
+
+    status = IoCallDriver(device, irp);
+    ok(status == STATUS_PENDING, "IoCallDriver returned %#x\n", status);
+
+    status = wait_single(&event, 0);
+    ok(status == STATUS_TIMEOUT, "got %#x\n", status);
+
+    irp->IoStatus.Status = STATUS_SUCCESS;
+    irp->IoStatus.Information = 0;
+    IoCompleteRequest(irp, IO_NO_INCREMENT);
+
+    status = wait_single(&event, 0);
+    ok(status == STATUS_SUCCESS, "got %#x\n", status);
+}
+
+static int cancel_cnt;
+
+static void WINAPI cancel_irp(DEVICE_OBJECT *device, IRP *irp)
+{
+    IoReleaseCancelSpinLock(irp->CancelIrql);
+    ok(irp->Cancel == TRUE, "Cancel = %x\n", irp->Cancel);
+    ok(!irp->CancelRoutine, "CancelRoutine = %p\n", irp->CancelRoutine);
+    irp->IoStatus.Status = STATUS_CANCELLED;
+    irp->IoStatus.Information = 0;
+    cancel_cnt++;
+}
+
+static void WINAPI cancel_ioctl_irp(DEVICE_OBJECT *device, IRP *irp)
+{
+    IoReleaseCancelSpinLock(irp->CancelIrql);
+    irp->IoStatus.Status = STATUS_CANCELLED;
+    irp->IoStatus.Information = 0;
+    cancel_cnt++;
+    IoCompleteRequest(irp, IO_NO_INCREMENT);
+}
+
+static NTSTATUS WINAPI cancel_test_completion(DEVICE_OBJECT *device, IRP *irp, void *context)
+{
+    ok(cancel_cnt == 1, "cancel_cnt = %d\n", cancel_cnt);
+    *(BOOL*)context = TRUE;
+    return STATUS_SUCCESS;
+}
+
+static void test_cancel_irp(DEVICE_OBJECT *device)
+{
+    IO_STACK_LOCATION *irpsp;
+    IO_STATUS_BLOCK iosb;
+    IRP *irp = NULL;
+    BOOL completion_called;
+    BOOLEAN r;
+    NTSTATUS status;
+
+    /* cancel IRP with no cancel routine */
+    irp = IoBuildAsynchronousFsdRequest(IRP_MJ_FLUSH_BUFFERS, device, NULL, 0, NULL, &iosb);
+
+    r = IoCancelIrp(irp);
+    ok(!r, "IoCancelIrp returned %x\n", r);
+    ok(irp->Cancel == TRUE, "Cancel = %x\n", irp->Cancel);
+
+    r = IoCancelIrp(irp);
+    ok(!r, "IoCancelIrp returned %x\n", r);
+    IoFreeIrp(irp);
+
+    irp = IoBuildAsynchronousFsdRequest(IRP_MJ_FLUSH_BUFFERS, device, NULL, 0, NULL, &iosb);
+
+    /* cancel IRP with cancel routine */
+    status = IoCallDriver(device, irp);
+    ok(status == STATUS_PENDING, "IoCallDriver returned %#x\n", status);
+
+    ok(irp->CurrentLocation == 1, "CurrentLocation = %u\n", irp->CurrentLocation);
+    irpsp = IoGetCurrentIrpStackLocation(irp);
+    ok(irpsp->DeviceObject == device, "DeviceObject = %u\n", irpsp->DeviceObject);
+
+    IoSetCancelRoutine(irp, cancel_irp);
+    cancel_cnt = 0;
+    r = IoCancelIrp(irp);
+    ok(r == TRUE, "IoCancelIrp returned %x\n", r);
+    ok(irp->Cancel == TRUE, "Cancel = %x\n", irp->Cancel);
+    ok(cancel_cnt == 1, "cancel_cnt = %d\n", cancel_cnt);
+
+    cancel_cnt = 0;
+    r = IoCancelIrp(irp);
+    ok(!r, "IoCancelIrp returned %x\n", r);
+    ok(irp->Cancel == TRUE, "Cancel = %x\n", irp->Cancel);
+    ok(!cancel_cnt, "cancel_cnt = %d\n", cancel_cnt);
+
+    IoCompleteRequest(irp, IO_NO_INCREMENT);
+
+    /* cancel IRP with cancel and completion routines with no SL_INVOKE_ON_ERROR */
+    irp = IoBuildAsynchronousFsdRequest(IRP_MJ_FLUSH_BUFFERS, device, NULL, 0, NULL, &iosb);
+    IoSetCompletionRoutine(irp, cancel_test_completion, &completion_called, TRUE, FALSE, TRUE);
+
+    status = IoCallDriver(device, irp);
+    ok(status == STATUS_PENDING, "IoCallDriver returned %#x\n", status);
+
+    IoSetCancelRoutine(irp, cancel_irp);
+    cancel_cnt = 0;
+    r = IoCancelIrp(irp);
+    ok(r == TRUE, "IoCancelIrp returned %x\n", r);
+    ok(cancel_cnt == 1, "cancel_cnt = %d\n", cancel_cnt);
+    ok(irp->Cancel == TRUE, "Cancel = %x\n", irp->Cancel);
+
+    completion_called = FALSE;
+    IoCompleteRequest(irp, IO_NO_INCREMENT);
+    ok(completion_called, "completion not called\n");
+
+    /* cancel IRP with cancel and completion routines with no SL_INVOKE_ON_CANCEL flag */
+    irp = IoBuildAsynchronousFsdRequest(IRP_MJ_FLUSH_BUFFERS, device, NULL, 0, NULL, &iosb);
+    IoSetCompletionRoutine(irp, cancel_test_completion, &completion_called, TRUE, TRUE, FALSE);
+
+    status = IoCallDriver(device, irp);
+    ok(status == STATUS_PENDING, "IoCallDriver returned %#x\n", status);
+
+    IoSetCancelRoutine(irp, cancel_irp);
+    cancel_cnt = 0;
+    r = IoCancelIrp(irp);
+    ok(r == TRUE, "IoCancelIrp returned %x\n", r);
+    ok(irp->Cancel == TRUE, "Cancel = %x\n", irp->Cancel);
+    ok(cancel_cnt == 1, "cancel_cnt = %d\n", cancel_cnt);
+
+    completion_called = FALSE;
+    IoCompleteRequest(irp, IO_NO_INCREMENT);
+    ok(completion_called, "completion not called\n");
+
+    /* cancel IRP with cancel and completion routines, but no SL_INVOKE_ON_ERROR nor SL_INVOKE_ON_CANCEL flag */
+    irp = IoBuildAsynchronousFsdRequest(IRP_MJ_FLUSH_BUFFERS, device, NULL, 0, NULL, &iosb);
+    IoSetCompletionRoutine(irp, cancel_test_completion, &completion_called, TRUE, FALSE, FALSE);
+
+    status = IoCallDriver(device, irp);
+    ok(status == STATUS_PENDING, "IoCallDriver returned %#x\n", status);
+
+    IoSetCancelRoutine(irp, cancel_irp);
+    cancel_cnt = 0;
+    r = IoCancelIrp(irp);
+    ok(r == TRUE, "IoCancelIrp returned %x\n", r);
+    ok(irp->Cancel == TRUE, "Cancel = %x\n", irp->Cancel);
+    ok(cancel_cnt == 1, "cancel_cnt = %d\n", cancel_cnt);
+
+    completion_called = FALSE;
+    IoCompleteRequest(irp, IO_NO_INCREMENT);
+    ok(!completion_called, "completion not called\n");
+}
+
 static int callout_cnt;
 
 static void WINAPI callout(void *parameter)
@@ -757,7 +1020,7 @@ static void test_ob_reference(const WCHAR *test_path)
 {
     POBJECT_TYPE (WINAPI *pObGetObjectType)(void*);
     OBJECT_ATTRIBUTES attr = { sizeof(attr) };
-    HANDLE event_handle, file_handle, file_handle2, thread_handle;
+    HANDLE event_handle, file_handle, file_handle2, thread_handle, handle;
     DISPATCHER_HEADER *header;
     FILE_OBJECT *file;
     void *obj1, *obj2;
@@ -841,7 +1104,6 @@ static void test_ob_reference(const WCHAR *test_path)
 
     status = ObReferenceObjectByHandle(file_handle2, SYNCHRONIZE, *pIoFileObjectType, KernelMode, &obj2, NULL);
     ok(!status, "ObReferenceObjectByHandle failed: %#x\n", status);
-    todo_wine
     ok(obj1 == obj2, "obj1 != obj2\n");
 
     file = obj1;
@@ -863,9 +1125,24 @@ static void test_ob_reference(const WCHAR *test_path)
     status = wait_single(header, 0);
     ok(status == 0 || status == STATUS_TIMEOUT, "got %#x\n", status);
 
-    ObDereferenceObject(obj1);
     ObDereferenceObject(obj2);
 
+    status = ObOpenObjectByPointer(obj1, OBJ_KERNEL_HANDLE, NULL, 0, NULL, KernelMode, &handle);
+    ok(status == STATUS_SUCCESS, "ObOpenObjectByPointer failed: %#x\n", status);
+
+    status = ZwClose(handle);
+    ok(!status, "ZwClose failed: %#x\n", status);
+
+    status = ObReferenceObjectByHandle(thread_handle, SYNCHRONIZE, *pPsThreadType, KernelMode, &obj2, NULL);
+    ok(!status, "ObReferenceObjectByHandle failed: %#x\n", status);
+    ok(obj1 == obj2, "obj1 != obj2\n");
+    ObDereferenceObject(obj2);
+
+    status = ObOpenObjectByPointer(obj1, OBJ_KERNEL_HANDLE, NULL, 0, *pIoFileObjectType, KernelMode, &handle);
+    ok(status == STATUS_OBJECT_TYPE_MISMATCH, "ObOpenObjectByPointer returned: %#x\n", status);
+
+    ObDereferenceObject(obj1);
+
     status = ZwClose(thread_handle);
     ok(!status, "ZwClose failed: %#x\n", status);
 
@@ -1167,7 +1444,106 @@ static void test_resource(void)
     ok(status == STATUS_SUCCESS, "got status %#x\n", status);
 }
 
-static NTSTATUS main_test(DEVICE_OBJECT *device, IRP *irp, IO_STACK_LOCATION *stack, ULONG_PTR *info)
+static void test_lookup_thread(void)
+{
+    NTSTATUS status;
+    PETHREAD thread = NULL;
+
+    status = PsLookupThreadByThreadId(PsGetCurrentThreadId(), &thread);
+    ok(!status, "PsLookupThreadByThreadId failed: %#x\n", status);
+    ok((PKTHREAD)thread == KeGetCurrentThread(), "thread != KeGetCurrentThread\n");
+    if (thread) ObDereferenceObject(thread);
+
+    status = PsLookupThreadByThreadId(NULL, &thread);
+    ok(status == STATUS_INVALID_CID || broken(status == STATUS_INVALID_PARAMETER) /* winxp */,
+       "PsLookupThreadByThreadId returned %#x\n", status);
+}
+
+static void test_IoAttachDeviceToDeviceStack(void)
+{
+    DEVICE_OBJECT *dev1, *dev2, *dev3, *ret;
+    NTSTATUS status;
+
+    status = IoCreateDevice(driver_obj, 0, NULL, FILE_DEVICE_UNKNOWN,
+            FILE_DEVICE_SECURE_OPEN, FALSE, &dev1);
+    ok(status == STATUS_SUCCESS, "IoCreateDevice failed\n");
+    status = IoCreateDevice(driver_obj, 0, NULL, FILE_DEVICE_UNKNOWN,
+            FILE_DEVICE_SECURE_OPEN, FALSE, &dev2);
+    ok(status == STATUS_SUCCESS, "IoCreateDevice failed\n");
+    status = IoCreateDevice(driver_obj, 0, NULL, FILE_DEVICE_UNKNOWN,
+            FILE_DEVICE_SECURE_OPEN, FALSE, &dev3);
+    ok(status == STATUS_SUCCESS, "IoCreateDevice failed\n");
+
+    /* TODO: initialize devices properly */
+    dev1->Flags &= ~DO_DEVICE_INITIALIZING;
+    dev2->Flags &= ~DO_DEVICE_INITIALIZING;
+
+    ret = IoAttachDeviceToDeviceStack(dev2, dev1);
+    ok(ret == dev1, "IoAttachDeviceToDeviceStack returned %p, expected %p\n", ret, dev1);
+    ok(dev1->AttachedDevice == dev2, "dev1->AttachedDevice = %p, expected %p\n",
+            dev1->AttachedDevice, dev2);
+    ok(!dev2->AttachedDevice, "dev2->AttachedDevice = %p\n", dev2->AttachedDevice);
+    ok(dev1->StackSize == 1, "dev1->StackSize = %d\n", dev1->StackSize);
+    ok(dev2->StackSize == 2, "dev2->StackSize = %d\n", dev2->StackSize);
+
+    ret = IoAttachDeviceToDeviceStack(dev3, dev1);
+    ok(ret == dev2, "IoAttachDeviceToDeviceStack returned %p, expected %p\n", ret, dev2);
+    ok(dev1->AttachedDevice == dev2, "dev1->AttachedDevice = %p, expected %p\n",
+            dev1->AttachedDevice, dev2);
+    ok(dev2->AttachedDevice == dev3, "dev2->AttachedDevice = %p, expected %p\n",
+            dev2->AttachedDevice, dev3);
+    ok(!dev3->AttachedDevice, "dev3->AttachedDevice = %p\n", dev3->AttachedDevice);
+    ok(dev1->StackSize == 1, "dev1->StackSize = %d\n", dev1->StackSize);
+    ok(dev2->StackSize == 2, "dev2->StackSize = %d\n", dev2->StackSize);
+    ok(dev3->StackSize == 3, "dev3->StackSize = %d\n", dev3->StackSize);
+
+    IoDetachDevice(dev1);
+    ok(!dev1->AttachedDevice, "dev1->AttachedDevice = %p\n", dev1->AttachedDevice);
+    ok(dev2->AttachedDevice == dev3, "dev2->AttachedDevice = %p\n", dev2->AttachedDevice);
+
+    IoDetachDevice(dev2);
+    ok(!dev2->AttachedDevice, "dev2->AttachedDevice = %p\n", dev2->AttachedDevice);
+    ok(dev1->StackSize == 1, "dev1->StackSize = %d\n", dev1->StackSize);
+    ok(dev2->StackSize == 2, "dev2->StackSize = %d\n", dev2->StackSize);
+    ok(dev3->StackSize == 3, "dev3->StackSize = %d\n", dev3->StackSize);
+
+    IoDeleteDevice(dev1);
+    IoDeleteDevice(dev2);
+    IoDeleteDevice(dev3);
+}
+
+static PIO_WORKITEM main_test_work_item;
+
+static void WINAPI main_test_task(DEVICE_OBJECT *device, void *context)
+{
+    IRP *irp = context;
+    void *buffer = irp->AssociatedIrp.SystemBuffer;
+
+    IoFreeWorkItem(main_test_work_item);
+    main_test_work_item = NULL;
+
+    test_current_thread(TRUE);
+    test_critical_region(FALSE);
+    test_call_driver(device);
+    test_cancel_irp(device);
+
+    /* print process report */
+    if (winetest_debug)
+    {
+        kprintf("%04x:ntoskrnl: %d tests executed (%d marked as todo, %d %s), %d skipped.\n",
+            PsGetCurrentProcessId(), successes + failures + todo_successes + todo_failures,
+            todo_successes, failures + todo_failures,
+            (failures + todo_failures != 1) ? "failures" : "failure", skipped );
+    }
+    ZwClose(okfile);
+
+    *((LONG *)buffer) = failures;
+    irp->IoStatus.Status = STATUS_SUCCESS;
+    irp->IoStatus.Information = sizeof(failures);
+    IoCompleteRequest(irp, IO_NO_INCREMENT);
+}
+
+static NTSTATUS main_test(DEVICE_OBJECT *device, IRP *irp, IO_STACK_LOCATION *stack)
 {
     ULONG length = stack->Parameters.DeviceIoControl.OutputBufferLength;
     void *buffer = irp->AssociatedIrp.SystemBuffer;
@@ -1199,8 +1575,12 @@ static NTSTATUS main_test(DEVICE_OBJECT *device, IRP *irp, IO_STACK_LOCATION *st
     pPsThreadType = get_proc_address("PsThreadType");
     ok(!!pPsThreadType, "IofileObjectType not found\n");
 
+    pPsInitialSystemProcess = get_proc_address("PsInitialSystemProcess");
+    ok(!!pPsInitialSystemProcess, "PsInitialSystemProcess not found\n");
+
     test_irp_struct(irp, device);
-    test_currentprocess();
+    test_current_thread(FALSE);
+    test_critical_region(TRUE);
     test_mdl_map();
     test_init_funcs();
     test_load_driver();
@@ -1210,19 +1590,16 @@ static NTSTATUS main_test(DEVICE_OBJECT *device, IRP *irp, IO_STACK_LOCATION *st
     test_lookaside_list();
     test_ob_reference(test_input->path);
     test_resource();
+    test_lookup_thread();
+    test_IoAttachDeviceToDeviceStack();
 
-    /* print process report */
-    if (winetest_debug)
-    {
-        kprintf("%04x:ntoskrnl: %d tests executed (%d marked as todo, %d %s), %d skipped.\n",
-            PsGetCurrentProcessId(), successes + failures + todo_successes + todo_failures,
-            todo_successes, failures + todo_failures,
-            (failures + todo_failures != 1) ? "failures" : "failure", skipped );
-    }
-    ZwClose(okfile);
-    *((LONG *)buffer) = failures;
-    *info = sizeof(failures);
-    return STATUS_SUCCESS;
+    if (main_test_work_item) return STATUS_UNEXPECTED_IO_ERROR;
+
+    main_test_work_item = IoAllocateWorkItem(lower_device);
+    ok(main_test_work_item != NULL, "main_test_work_item = NULL\n");
+
+    IoQueueWorkItem(main_test_work_item, main_test_task, DelayedWorkQueue, irp);
+    return STATUS_PENDING;
 }
 
 static NTSTATUS test_basic_ioctl(IRP *irp, IO_STACK_LOCATION *stack, ULONG_PTR *info)
@@ -1242,6 +1619,22 @@ static NTSTATUS test_basic_ioctl(IRP *irp, IO_STACK_LOCATION *stack, ULONG_PTR *
     return STATUS_SUCCESS;
 }
 
+static NTSTATUS get_cancel_count(IRP *irp, IO_STACK_LOCATION *stack, ULONG_PTR *info)
+{
+    ULONG length = stack->Parameters.DeviceIoControl.OutputBufferLength;
+    char *buffer = irp->AssociatedIrp.SystemBuffer;
+
+    if (!buffer)
+        return STATUS_ACCESS_VIOLATION;
+
+    if (length < sizeof(DWORD))
+        return STATUS_BUFFER_TOO_SMALL;
+
+    *(DWORD*)buffer = cancel_cnt;
+    *info = sizeof(DWORD);
+    return STATUS_SUCCESS;
+}
+
 static NTSTATUS test_load_driver_ioctl(IRP *irp, IO_STACK_LOCATION *stack, ULONG_PTR *info)
 {
     BOOL *load = irp->AssociatedIrp.SystemBuffer;
@@ -1264,6 +1657,7 @@ static NTSTATUS WINAPI driver_Create(DEVICE_OBJECT *device, IRP *irp)
     IO_STACK_LOCATION *irpsp = IoGetCurrentIrpStackLocation( irp );
 
     last_created_file = irpsp->FileObject;
+    create_caller_thread = KeGetCurrentThread();
 
     irp->IoStatus.Status = STATUS_SUCCESS;
     IoCompleteRequest(irp, IO_NO_INCREMENT);
@@ -1281,20 +1675,50 @@ static NTSTATUS WINAPI driver_IoControl(DEVICE_OBJECT *device, IRP *irp)
             status = test_basic_ioctl(irp, stack, &irp->IoStatus.Information);
             break;
         case IOCTL_WINETEST_MAIN_TEST:
-            status = main_test(device, irp, stack, &irp->IoStatus.Information);
+            status = main_test(device, irp, stack);
             break;
         case IOCTL_WINETEST_LOAD_DRIVER:
             status = test_load_driver_ioctl(irp, stack, &irp->IoStatus.Information);
             break;
+        case IOCTL_WINETEST_RESET_CANCEL:
+            cancel_cnt = 0;
+            status = STATUS_SUCCESS;
+            break;
+        case IOCTL_WINETEST_TEST_CANCEL:
+            IoSetCancelRoutine(irp, cancel_ioctl_irp);
+            IoMarkIrpPending(irp);
+            return STATUS_PENDING;
+        case IOCTL_WINETEST_GET_CANCEL_COUNT:
+            status = get_cancel_count(irp, stack, &irp->IoStatus.Information);
+            break;
+        case IOCTL_WINETEST_DETACH:
+            IoDetachDevice(lower_device);
+            status = STATUS_SUCCESS;
+            break;
         default:
             break;
     }
 
-    irp->IoStatus.Status = status;
-    IoCompleteRequest(irp, IO_NO_INCREMENT);
+    if (status != STATUS_PENDING)
+    {
+        irp->IoStatus.Status = status;
+        IoCompleteRequest(irp, IO_NO_INCREMENT);
+    }
+    else IoMarkIrpPending(irp);
     return status;
 }
 
+static NTSTATUS WINAPI driver_FlushBuffers(DEVICE_OBJECT *device, IRP *irp)
+{
+    IO_STACK_LOCATION *irpsp = IoGetCurrentIrpStackLocation(irp);
+    ok(device == lower_device, "Expected device %p, got %p.\n", lower_device, device);
+    ok(irpsp->DeviceObject == device, "device != DeviceObject\n");
+    ok(irp->Tail.Overlay.Thread == (PETHREAD)KeGetCurrentThread(),
+       "IRP thread is not the current thread\n");
+    IoMarkIrpPending(irp);
+    return STATUS_PENDING;
+}
+
 static NTSTATUS WINAPI driver_Close(DEVICE_OBJECT *device, IRP *irp)
 {
     irp->IoStatus.Status = STATUS_SUCCESS;
@@ -1311,31 +1735,51 @@ static VOID WINAPI driver_Unload(DRIVER_OBJECT *driver)
     RtlInitUnicodeString(&linkW, driver_link);
     IoDeleteSymbolicLink(&linkW);
 
-    IoDeleteDevice(driver->DeviceObject);
+    IoDeleteDevice(upper_device);
+    IoDeleteDevice(lower_device);
 }
 
 NTSTATUS WINAPI DriverEntry(DRIVER_OBJECT *driver, PUNICODE_STRING registry)
 {
     UNICODE_STRING nameW, linkW;
-    DEVICE_OBJECT *device;
     NTSTATUS status;
 
     DbgPrint("loading driver\n");
 
+    driver_obj = driver;
+
     /* Allow unloading of the driver */
     driver->DriverUnload = driver_Unload;
 
     /* Set driver functions */
     driver->MajorFunction[IRP_MJ_CREATE]            = driver_Create;
     driver->MajorFunction[IRP_MJ_DEVICE_CONTROL]    = driver_IoControl;
+    driver->MajorFunction[IRP_MJ_FLUSH_BUFFERS]     = driver_FlushBuffers;
     driver->MajorFunction[IRP_MJ_CLOSE]             = driver_Close;
 
-    RtlInitUnicodeString(&nameW, driver_device);
+    RtlInitUnicodeString(&nameW, device_name);
     RtlInitUnicodeString(&linkW, driver_link);
 
     if (!(status = IoCreateDevice(driver, 0, &nameW, FILE_DEVICE_UNKNOWN,
-                                  FILE_DEVICE_SECURE_OPEN, FALSE, &device)))
+                                  FILE_DEVICE_SECURE_OPEN, FALSE, &lower_device)))
+    {
         status = IoCreateSymbolicLink(&linkW, &nameW);
+        lower_device->Flags &= ~DO_DEVICE_INITIALIZING;
+    }
+
+    if (!status)
+    {
+        RtlInitUnicodeString(&nameW, upper_name);
+
+        status = IoCreateDevice(driver, 0, &nameW, FILE_DEVICE_UNKNOWN,
+                                FILE_DEVICE_SECURE_OPEN, FALSE, &upper_device);
+    }
+
+    if (!status)
+    {
+        IoAttachDeviceToDeviceStack(upper_device, lower_device);
+        upper_device->Flags &= ~DO_DEVICE_INITIALIZING;
+    }
 
     return status;
 }
