@@ -116,33 +116,70 @@ static inline AviMux* impl_from_BaseFilter(BaseFilter *filter)
     return CONTAINING_RECORD(filter, AviMux, filter);
 }
 
-static IPin* WINAPI AviMux_GetPin(BaseFilter *iface, int pos)
+static IPin *avi_mux_get_pin(BaseFilter *iface, unsigned int index)
 {
     AviMux *This = impl_from_BaseFilter(iface);
 
-    TRACE("(%p)->(%d)\n", This, pos);
-
-    if(pos == 0) {
+    if (!index)
+    {
         IPin_AddRef(&This->out->pin.pin.IPin_iface);
         return &This->out->pin.pin.IPin_iface;
-    }else if(pos>0 && pos<=This->input_pin_no) {
-        IPin_AddRef(&This->in[pos-1]->pin.pin.IPin_iface);
-        return &This->in[pos-1]->pin.pin.IPin_iface;
+    }
+    else if (index <= This->input_pin_no)
+    {
+        IPin_AddRef(&This->in[index - 1]->pin.pin.IPin_iface);
+        return &This->in[index - 1]->pin.pin.IPin_iface;
     }
 
     return NULL;
 }
 
-static LONG WINAPI AviMux_GetPinCount(BaseFilter *iface)
+static void avi_mux_destroy(BaseFilter *iface)
 {
-    AviMux *This = impl_from_BaseFilter(iface);
-    TRACE("(%p)\n", This);
-    return This->input_pin_no+1;
+    AviMux *filter = impl_from_BaseFilter(iface);
+    int i;
+
+    BaseOutputPinImpl_Release(&filter->out->pin.pin.IPin_iface);
+
+    for (i = 0; i < filter->input_pin_no; ++i)
+    {
+        IPin_Disconnect(&filter->in[i]->pin.pin.IPin_iface);
+        IMemAllocator_Release(filter->in[i]->samples_allocator);
+        filter->in[i]->samples_allocator = NULL;
+        BaseInputPinImpl_Release(&filter->in[i]->pin.pin.IPin_iface);
+    }
+
+    heap_free(filter->idx1);
+    strmbase_filter_cleanup(&filter->filter);
+    heap_free(filter);
+    ObjectRefCount(FALSE);
+}
+
+static HRESULT avi_mux_query_interface(BaseFilter *iface, REFIID iid, void **out)
+{
+    AviMux *filter = impl_from_BaseFilter(iface);
+
+    if (IsEqualGUID(iid, &IID_IConfigAviMux))
+        *out = &filter->IConfigAviMux_iface;
+    else if (IsEqualGUID(iid, &IID_IConfigInterleaving))
+        *out = &filter->IConfigInterleaving_iface;
+    else if (IsEqualGUID(iid, &IID_IMediaSeeking))
+        *out = &filter->IMediaSeeking_iface;
+    else if (IsEqualGUID(iid, &IID_IPersistMediaPropertyBag))
+        *out = &filter->IPersistMediaPropertyBag_iface;
+    else if (IsEqualGUID(iid, &IID_ISpecifyPropertyPages))
+        *out = &filter->ISpecifyPropertyPages_iface;
+    else
+        return E_NOINTERFACE;
+
+    IUnknown_AddRef((IUnknown *)*out);
+    return S_OK;
 }
 
 static const BaseFilterFuncTable filter_func_table = {
-    AviMux_GetPin,
-    AviMux_GetPinCount
+    .filter_get_pin = avi_mux_get_pin,
+    .filter_destroy = avi_mux_destroy,
+    .filter_query_interface = avi_mux_query_interface,
 };
 
 static inline AviMux* impl_from_IBaseFilter(IBaseFilter *iface)
@@ -151,61 +188,6 @@ static inline AviMux* impl_from_IBaseFilter(IBaseFilter *iface)
     return impl_from_BaseFilter(filter);
 }
 
-static HRESULT WINAPI AviMux_QueryInterface(IBaseFilter *iface, REFIID riid, void **ppv)
-{
-    AviMux *This = impl_from_IBaseFilter(iface);
-
-    TRACE("(%p)->(%s %p)\n", This, debugstr_guid(riid), ppv);
-
-    if(IsEqualIID(riid, &IID_IUnknown) || IsEqualIID(riid, &IID_IPersist) ||
-            IsEqualIID(riid, &IID_IMediaFilter) || IsEqualIID(riid, &IID_IBaseFilter))
-        *ppv = &This->filter.IBaseFilter_iface;
-    else if(IsEqualIID(riid, &IID_IConfigAviMux))
-        *ppv = &This->IConfigAviMux_iface;
-    else if(IsEqualIID(riid, &IID_IConfigInterleaving))
-        *ppv = &This->IConfigInterleaving_iface;
-    else if(IsEqualIID(riid, &IID_IMediaSeeking))
-        *ppv = &This->IMediaSeeking_iface;
-    else if(IsEqualIID(riid, &IID_IPersistMediaPropertyBag))
-        *ppv = &This->IPersistMediaPropertyBag_iface;
-    else if(IsEqualIID(riid, &IID_ISpecifyPropertyPages))
-        *ppv = &This->ISpecifyPropertyPages_iface;
-    else {
-        FIXME("no interface for %s\n", debugstr_guid(riid));
-        *ppv = NULL;
-        return E_NOINTERFACE;
-    }
-
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
-}
-
-static ULONG WINAPI AviMux_Release(IBaseFilter *iface)
-{
-    AviMux *This = impl_from_IBaseFilter(iface);
-    ULONG ref = BaseFilterImpl_Release(iface);
-
-    TRACE("(%p) new refcount: %u\n", This, ref);
-
-    if(!ref) {
-        int i;
-
-        BaseOutputPinImpl_Release(&This->out->pin.pin.IPin_iface);
-
-        for(i=0; i<This->input_pin_no; i++) {
-            IPin_Disconnect(&This->in[i]->pin.pin.IPin_iface);
-            IMemAllocator_Release(This->in[i]->samples_allocator);
-            This->in[i]->samples_allocator = NULL;
-            BaseInputPinImpl_Release(&This->in[i]->pin.pin.IPin_iface);
-        }
-
-        HeapFree(GetProcessHeap(), 0, This->idx1);
-        HeapFree(GetProcessHeap(), 0, This);
-        ObjectRefCount(FALSE);
-    }
-    return ref;
-}
-
 static HRESULT out_flush(AviMux *This)
 {
     ULONG written;
@@ -600,7 +582,7 @@ static HRESULT WINAPI AviMux_Run(IBaseFilter *iface, REFERENCE_TIME tStart)
     HRESULT hr;
     int i, stream_id;
 
-    TRACE("(%p)->(0x%x%08x)\n", This, (ULONG)(tStart >> 32), (ULONG)tStart);
+    TRACE("(%p)->(%s)\n", This, wine_dbgstr_longlong(tStart));
 
     if(This->filter.state == State_Running)
         return S_OK;
@@ -720,59 +702,10 @@ static HRESULT WINAPI AviMux_Run(IBaseFilter *iface, REFERENCE_TIME tStart)
     return S_OK;
 }
 
-static HRESULT WINAPI AviMux_EnumPins(IBaseFilter *iface, IEnumPins **ppEnum)
-{
-    AviMux *This = impl_from_IBaseFilter(iface);
-    TRACE("(%p)->(%p)\n", This, ppEnum);
-    return BaseFilterImpl_EnumPins(iface, ppEnum);
-}
-
-static HRESULT WINAPI AviMux_FindPin(IBaseFilter *iface, LPCWSTR Id, IPin **ppPin)
-{
-    AviMux *This = impl_from_IBaseFilter(iface);
-    int i;
-
-    TRACE("(%p)->(%s %p)\n", This, debugstr_w(Id), ppPin);
-
-    if(!Id || !ppPin)
-        return E_POINTER;
-
-    if(!lstrcmpiW(Id, This->out->pin.pin.pinInfo.achName)) {
-        IPin_AddRef(&This->out->pin.pin.IPin_iface);
-        *ppPin = &This->out->pin.pin.IPin_iface;
-        return S_OK;
-    }
-
-    for(i=0; i<This->input_pin_no; i++) {
-        if(lstrcmpiW(Id, This->in[i]->pin.pin.pinInfo.achName))
-            continue;
-
-        IPin_AddRef(&This->in[i]->pin.pin.IPin_iface);
-        *ppPin = &This->in[i]->pin.pin.IPin_iface;
-        return S_OK;
-    }
-
-    return VFW_E_NOT_FOUND;
-}
-
-static HRESULT WINAPI AviMux_QueryFilterInfo(IBaseFilter *iface, FILTER_INFO *pInfo)
-{
-    AviMux *This = impl_from_IBaseFilter(iface);
-    FIXME("(%p)->(%p)\n", This, pInfo);
-    return E_NOTIMPL;
-}
-
-static HRESULT WINAPI AviMux_QueryVendorInfo(IBaseFilter *iface, LPWSTR *pVendorInfo)
-{
-    AviMux *This = impl_from_IBaseFilter(iface);
-    FIXME("(%p)->(%p)\n", This, pVendorInfo);
-    return E_NOTIMPL;
-}
-
 static const IBaseFilterVtbl AviMuxVtbl = {
-    AviMux_QueryInterface,
+    BaseFilterImpl_QueryInterface,
     BaseFilterImpl_AddRef,
-    AviMux_Release,
+    BaseFilterImpl_Release,
     BaseFilterImpl_GetClassID,
     AviMux_Stop,
     AviMux_Pause,
@@ -780,11 +713,11 @@ static const IBaseFilterVtbl AviMuxVtbl = {
     BaseFilterImpl_GetState,
     BaseFilterImpl_SetSyncSource,
     BaseFilterImpl_GetSyncSource,
-    AviMux_EnumPins,
-    AviMux_FindPin,
-    AviMux_QueryFilterInfo,
+    BaseFilterImpl_EnumPins,
+    BaseFilterImpl_FindPin,
+    BaseFilterImpl_QueryFilterInfo,
     BaseFilterImpl_JoinFilterGraph,
-    AviMux_QueryVendorInfo
+    BaseFilterImpl_QueryVendorInfo
 };
 
 static inline AviMux* impl_from_IConfigAviMux(IConfigAviMux *iface)
@@ -1223,7 +1156,13 @@ static const ISpecifyPropertyPagesVtbl SpecifyPropertyPagesVtbl = {
     SpecifyPropertyPages_GetPages
 };
 
-static HRESULT WINAPI AviMuxOut_AttemptConnection(BasePin *base,
+static HRESULT WINAPI AviMuxOut_CheckMediaType(BasePin *base, const AM_MEDIA_TYPE *amt)
+{
+    FIXME("(%p) stub\n", base);
+    return S_OK;
+}
+
+static HRESULT WINAPI AviMuxOut_AttemptConnection(BaseOutputPin *base,
         IPin *pReceivePin, const AM_MEDIA_TYPE *pmt)
 {
     PIN_DIRECTION dir;
@@ -1239,11 +1178,6 @@ static HRESULT WINAPI AviMuxOut_AttemptConnection(BasePin *base,
     return BaseOutputPinImpl_AttemptConnection(base, pReceivePin, pmt);
 }
 
-static LONG WINAPI AviMuxOut_GetMediaTypeVersion(BasePin *base)
-{
-    return 0;
-}
-
 static HRESULT WINAPI AviMuxOut_GetMediaType(BasePin *base, int iPosition, AM_MEDIA_TYPE *amt)
 {
     TRACE("(%p)->(%d %p)\n", base, iPosition, amt);
@@ -1291,22 +1225,14 @@ static HRESULT WINAPI AviMuxOut_DecideAllocator(BaseOutputPin *base,
     return IMemInputPin_NotifyAllocator(pPin, *pAlloc, TRUE);
 }
 
-static HRESULT WINAPI AviMuxOut_BreakConnect(BaseOutputPin *base)
-{
-    FIXME("(%p)\n", base);
-    return E_NOTIMPL;
-}
-
 static const BaseOutputPinFuncTable AviMuxOut_BaseOutputFuncTable = {
     {
-        NULL,
-        AviMuxOut_AttemptConnection,
-        AviMuxOut_GetMediaTypeVersion,
+        AviMuxOut_CheckMediaType,
         AviMuxOut_GetMediaType
     },
+    AviMuxOut_AttemptConnection,
     NULL,
     AviMuxOut_DecideAllocator,
-    AviMuxOut_BreakConnect
 };
 
 static inline AviMux* impl_from_out_IPin(IPin *iface)
@@ -1484,8 +1410,7 @@ static HRESULT WINAPI AviMuxOut_NewSegment(IPin *iface, REFERENCE_TIME tStart,
         REFERENCE_TIME tStop, double dRate)
 {
     AviMux *This = impl_from_out_IPin(iface);
-    TRACE("(%p)->(0x%x%08x 0x%x%08x %lf)\n", This, (ULONG)(tStart >> 32),
-            (ULONG)tStart, (ULONG)(tStop >> 32), (ULONG)tStop, dRate);
+    TRACE("(%p)->(%s %s %f)\n", This, wine_dbgstr_longlong(tStart), wine_dbgstr_longlong(tStop), dRate);
     return BasePinImpl_NewSegment(iface, tStart, tStop, dRate);
 }
 
@@ -1580,11 +1505,6 @@ static HRESULT WINAPI AviMuxIn_CheckMediaType(BasePin *base, const AM_MEDIA_TYPE
     return S_FALSE;
 }
 
-static LONG WINAPI AviMuxIn_GetMediaTypeVersion(BasePin *base)
-{
-    return 0;
-}
-
 static HRESULT WINAPI AviMuxIn_GetMediaType(BasePin *base, int iPosition, AM_MEDIA_TYPE *amt)
 {
     return S_FALSE;
@@ -1696,8 +1616,6 @@ static HRESULT WINAPI AviMuxIn_Receive(BaseInputPin *base, IMediaSample *pSample
 static const BaseInputPinFuncTable AviMuxIn_BaseInputFuncTable = {
     {
         AviMuxIn_CheckMediaType,
-        NULL,
-        AviMuxIn_GetMediaTypeVersion,
         AviMuxIn_GetMediaType
     },
     AviMuxIn_Receive
@@ -1960,8 +1878,8 @@ static HRESULT WINAPI AviMuxIn_NewSegment(IPin *iface, REFERENCE_TIME tStart,
 {
     AviMux *This = impl_from_in_IPin(iface);
     AviMuxIn *avimuxin = AviMuxIn_from_IPin(iface);
-    TRACE("(%p:%s)->(0x%x%08x 0x%x%08x %lf)\n", This, debugstr_w(avimuxin->pin.pin.pinInfo.achName),
-         (ULONG)(tStart >> 32), (ULONG)tStart, (ULONG)(tStop >> 32), (ULONG)tStop, dRate);
+    TRACE("(%p:%s)->(%s %s %f)\n", This, debugstr_w(avimuxin->pin.pin.pinInfo.achName),
+         wine_dbgstr_longlong(tStart), wine_dbgstr_longlong(tStop), dRate);
     return BasePinImpl_NewSegment(iface, tStart, tStop, dRate);
 }
 
@@ -2352,7 +2270,7 @@ static HRESULT create_input_pin(AviMux *avimux)
     return S_OK;
 }
 
-IUnknown* WINAPI QCAP_createAVIMux(IUnknown *pUnkOuter, HRESULT *phr)
+IUnknown * WINAPI QCAP_createAVIMux(IUnknown *outer, HRESULT *phr)
 {
     static const WCHAR output_name[] = {'A','V','I',' ','O','u','t',0};
 
@@ -2360,20 +2278,13 @@ IUnknown* WINAPI QCAP_createAVIMux(IUnknown *pUnkOuter, HRESULT *phr)
     PIN_INFO info;
     HRESULT hr;
 
-    TRACE("(%p)\n", pUnkOuter);
-
-    if(pUnkOuter) {
-        *phr = CLASS_E_NOAGGREGATION;
-        return NULL;
-    }
-
     avimux = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(AviMux));
     if(!avimux) {
         *phr = E_OUTOFMEMORY;
         return NULL;
     }
 
-    BaseFilter_Init(&avimux->filter, &AviMuxVtbl, &CLSID_AviDest,
+    strmbase_filter_init(&avimux->filter, &AviMuxVtbl, outer, &CLSID_AviDest,
             (DWORD_PTR)(__FILE__ ": AviMux.csFilter"), &filter_func_table);
     avimux->IConfigAviMux_iface.lpVtbl = &ConfigAviMuxVtbl;
     avimux->IConfigInterleaving_iface.lpVtbl = &ConfigInterleavingVtbl;
@@ -2387,7 +2298,7 @@ IUnknown* WINAPI QCAP_createAVIMux(IUnknown *pUnkOuter, HRESULT *phr)
     hr = BaseOutputPin_Construct(&AviMuxOut_PinVtbl, sizeof(AviMuxOut), &info,
             &AviMuxOut_BaseOutputFuncTable, &avimux->filter.csFilter, (IPin**)&avimux->out);
     if(FAILED(hr)) {
-        BaseFilterImpl_Release(&avimux->filter.IBaseFilter_iface);
+        strmbase_filter_cleanup(&avimux->filter);
         HeapFree(GetProcessHeap(), 0, avimux);
         *phr = hr;
         return NULL;
@@ -2400,7 +2311,7 @@ IUnknown* WINAPI QCAP_createAVIMux(IUnknown *pUnkOuter, HRESULT *phr)
     hr = create_input_pin(avimux);
     if(FAILED(hr)) {
         BaseOutputPinImpl_Release(&avimux->out->pin.pin.IPin_iface);
-        BaseFilterImpl_Release(&avimux->filter.IBaseFilter_iface);
+        strmbase_filter_cleanup(&avimux->filter);
         HeapFree(GetProcessHeap(), 0, avimux);
         *phr = hr;
         return NULL;
@@ -2410,5 +2321,5 @@ IUnknown* WINAPI QCAP_createAVIMux(IUnknown *pUnkOuter, HRESULT *phr)
 
     ObjectRefCount(TRUE);
     *phr = S_OK;
-    return (IUnknown*)&avimux->filter.IBaseFilter_iface;
+    return &avimux->filter.IUnknown_inner;
 }
