@@ -37,6 +37,19 @@ WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 
 static const WCHAR wszOutputPinName[] = { 'O','u','t','p','u','t',0 };
 
+static const AM_MEDIA_TYPE default_mt =
+{
+    {0xe436eb83,0x524f,0x11ce,{0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70}},   /* MEDIATYPE_Stream */
+    {0,0,0,{0,0,0,0,0,0,0,0}},
+    TRUE,
+    FALSE,
+    1,
+    {0,0,0,{0,0,0,0,0,0,0,0}},
+    NULL,
+    0,
+    NULL
+};
+
 typedef struct AsyncReader
 {
     BaseFilter filter;
@@ -87,8 +100,7 @@ static HRESULT process_extensions(HKEY hkeyExtensions, LPCOLESTR pszFileName, GU
         return E_POINTER;
 
     /* Get the part of the name that matters */
-    extension = PathFindExtensionW(pszFileName);
-    if (*extension != '.')
+    if (!(extension = strrchrW(pszFileName, '.')))
         return E_FAIL;
 
     l = RegOpenKeyExW(hkeyExtensions, extension, 0, KEY_READ, &hsub);
@@ -380,49 +392,71 @@ HRESULT GetClassMediaFile(IAsyncReader * pReader, LPCOLESTR pszFileName, GUID *
     return hr;
 }
 
-static IPin* WINAPI AsyncReader_GetPin(BaseFilter *iface, int pos)
+static IPin *async_reader_get_pin(BaseFilter *iface, unsigned int index)
 {
     AsyncReader *This = impl_from_BaseFilter(iface);
 
-    TRACE("%p->(%d)\n", This, pos);
-
-    if (pos >= 1 || !This->pOutputPin)
+    if (index >= 1 || !This->pOutputPin)
         return NULL;
 
     IPin_AddRef(This->pOutputPin);
     return This->pOutputPin;
 }
 
-static LONG WINAPI AsyncReader_GetPinCount(BaseFilter *iface)
+static void async_reader_destroy(BaseFilter *iface)
 {
-    AsyncReader *This = impl_from_BaseFilter(iface);
+    AsyncReader *filter = impl_from_BaseFilter(iface);
 
-    TRACE("%p->()\n", This);
-
-    if (!This->pOutputPin)
-        return 0;
-    else
-        return 1;
+    if (filter->pOutputPin)
+    {
+        IPin *peer;
+        if (SUCCEEDED(IPin_ConnectedTo(filter->pOutputPin, &peer)))
+        {
+            IPin_Disconnect(peer);
+            IPin_Release(peer);
+        }
+        IPin_Disconnect(filter->pOutputPin);
+        IPin_Release(filter->pOutputPin);
+    }
+    CoTaskMemFree(filter->pszFileName);
+    if (filter->pmt)
+        DeleteMediaType(filter->pmt);
+    strmbase_filter_cleanup(&filter->filter);
+    CoTaskMemFree(filter);
 }
 
-static const BaseFilterFuncTable BaseFuncTable = {
-    AsyncReader_GetPin,
-    AsyncReader_GetPinCount
+static HRESULT async_reader_query_interface(BaseFilter *iface, REFIID iid, void **out)
+{
+    AsyncReader *filter = impl_from_BaseFilter(iface);
+
+    if (IsEqualGUID(iid, &IID_IFileSourceFilter))
+    {
+        *out = &filter->IFileSourceFilter_iface;
+        IUnknown_AddRef((IUnknown *)*out);
+        return S_OK;
+    }
+
+    return E_NOINTERFACE;
+}
+
+static const BaseFilterFuncTable BaseFuncTable =
+{
+    .filter_get_pin = async_reader_get_pin,
+    .filter_destroy = async_reader_destroy,
+    .filter_query_interface = async_reader_query_interface,
 };
 
-HRESULT AsyncReader_create(IUnknown * pUnkOuter, LPVOID * ppv)
+HRESULT AsyncReader_create(IUnknown *outer, void **out)
 {
     AsyncReader *pAsyncRead;
     
-    if( pUnkOuter )
-        return CLASS_E_NOAGGREGATION;
-    
     pAsyncRead = CoTaskMemAlloc(sizeof(AsyncReader));
 
     if (!pAsyncRead)
         return E_OUTOFMEMORY;
 
-    BaseFilter_Init(&pAsyncRead->filter, &AsyncReader_Vtbl, &CLSID_AsyncReader, (DWORD_PTR)(__FILE__ ": AsyncReader.csFilter"), &BaseFuncTable);
+    strmbase_filter_init(&pAsyncRead->filter, &AsyncReader_Vtbl, outer, &CLSID_AsyncReader,
+            (DWORD_PTR)(__FILE__ ": AsyncReader.csFilter"), &BaseFuncTable);
 
     pAsyncRead->IFileSourceFilter_iface.lpVtbl = &FileSource_Vtbl;
     pAsyncRead->pOutputPin = NULL;
@@ -430,78 +464,13 @@ HRESULT AsyncReader_create(IUnknown * pUnkOuter, LPVOID * ppv)
     pAsyncRead->pszFileName = NULL;
     pAsyncRead->pmt = NULL;
 
-    *ppv = pAsyncRead;
+    *out = &pAsyncRead->filter.IUnknown_inner;
 
     TRACE("-- created at %p\n", pAsyncRead);
 
     return S_OK;
 }
 
-/** IUnknown methods **/
-
-static HRESULT WINAPI AsyncReader_QueryInterface(IBaseFilter * iface, REFIID riid, LPVOID * ppv)
-{
-    AsyncReader *This = impl_from_IBaseFilter(iface);
-
-    TRACE("%p->(%s, %p)\n", This, qzdebugstr_guid(riid), ppv);
-
-    *ppv = NULL;
-
-    if (IsEqualIID(riid, &IID_IUnknown))
-        *ppv = &This->filter.IBaseFilter_iface;
-    else if (IsEqualIID(riid, &IID_IPersist))
-        *ppv = &This->filter.IBaseFilter_iface;
-    else if (IsEqualIID(riid, &IID_IMediaFilter))
-        *ppv = &This->filter.IBaseFilter_iface;
-    else if (IsEqualIID(riid, &IID_IBaseFilter))
-        *ppv = &This->filter.IBaseFilter_iface;
-    else if (IsEqualIID(riid, &IID_IFileSourceFilter))
-        *ppv = &This->IFileSourceFilter_iface;
-
-    if (*ppv)
-    {
-        IUnknown_AddRef((IUnknown *)(*ppv));
-        return S_OK;
-    }
-
-    if (!IsEqualIID(riid, &IID_IPin) && !IsEqualIID(riid, &IID_IMediaSeeking) &&
-        !IsEqualIID(riid, &IID_IVideoWindow) && !IsEqualIID(riid, &IID_IBasicAudio))
-        FIXME("No interface for %s!\n", qzdebugstr_guid(riid));
-
-    return E_NOINTERFACE;
-}
-
-static ULONG WINAPI AsyncReader_Release(IBaseFilter * iface)
-{
-    AsyncReader *This = impl_from_IBaseFilter(iface);
-    ULONG refCount = InterlockedDecrement(&This->filter.refCount);
-    
-    TRACE("%p->() Release from %d\n", This, refCount + 1);
-    
-    if (!refCount)
-    {
-        if (This->pOutputPin)
-        {
-            IPin *pConnectedTo;
-            if(SUCCEEDED(IPin_ConnectedTo(This->pOutputPin, &pConnectedTo)))
-            {
-                IPin_Disconnect(pConnectedTo);
-                IPin_Release(pConnectedTo);
-            }
-            IPin_Disconnect(This->pOutputPin);
-            IPin_Release(This->pOutputPin);
-        }
-        CoTaskMemFree(This->pszFileName);
-        if (This->pmt)
-            DeleteMediaType(This->pmt);
-        BaseFilter_Destroy(&This->filter);
-        CoTaskMemFree(This);
-        return 0;
-    }
-    else
-        return refCount;
-}
-
 /** IMediaFilter methods **/
 
 static HRESULT WINAPI AsyncReader_Stop(IBaseFilter * iface)
@@ -539,9 +508,9 @@ static HRESULT WINAPI AsyncReader_Run(IBaseFilter * iface, REFERENCE_TIME tStart
 
 static const IBaseFilterVtbl AsyncReader_Vtbl =
 {
-    AsyncReader_QueryInterface,
+    BaseFilterImpl_QueryInterface,
     BaseFilterImpl_AddRef,
-    AsyncReader_Release,
+    BaseFilterImpl_Release,
     BaseFilterImpl_GetClassID,
     AsyncReader_Stop,
     AsyncReader_Pause,
@@ -618,19 +587,11 @@ static HRESULT WINAPI FileSource_Load(IFileSourceFilter * iface, LPCOLESTR pszFi
         This->pmt = CoTaskMemAlloc(sizeof(AM_MEDIA_TYPE));
         if (!pmt)
         {
-            This->pmt->bFixedSizeSamples = TRUE;
-            This->pmt->bTemporalCompression = FALSE;
-            This->pmt->cbFormat = 0;
-            This->pmt->pbFormat = NULL;
-            This->pmt->pUnk = NULL;
-            This->pmt->lSampleSize = 0;
-            This->pmt->formattype = FORMAT_None;
-            hr = GetClassMediaFile(pReader, pszFileName, &This->pmt->majortype, &This->pmt->subtype, NULL);
-            if (FAILED(hr))
+            CopyMediaType(This->pmt, &default_mt);
+            if (FAILED(GetClassMediaFile(pReader, pszFileName, &This->pmt->majortype, &This->pmt->subtype, NULL)))
             {
                 This->pmt->majortype = MEDIATYPE_Stream;
                 This->pmt->subtype = MEDIASUBTYPE_NULL;
-                hr = S_OK;
             }
         }
         else
@@ -756,24 +717,27 @@ static HRESULT WINAPI FileAsyncReaderPin_CheckMediaType(BasePin *pin, const AM_M
 {
     AM_MEDIA_TYPE *pmt_filter = impl_from_IBaseFilter(pin->pinInfo.pFilter)->pmt;
 
-    FIXME("(%p, %p)\n", pin, pmt);
-
     if (IsEqualGUID(&pmt->majortype, &pmt_filter->majortype) &&
-        IsEqualGUID(&pmt->subtype, &pmt_filter->subtype) &&
-        IsEqualGUID(&pmt->formattype, &FORMAT_None))
+        IsEqualGUID(&pmt->subtype, &pmt_filter->subtype))
         return S_OK;
 
     return S_FALSE;
 }
 
-static HRESULT WINAPI FileAsyncReaderPin_GetMediaType(BasePin *iface, int iPosition, AM_MEDIA_TYPE *pmt)
+static HRESULT WINAPI FileAsyncReaderPin_GetMediaType(BasePin *iface, int index, AM_MEDIA_TYPE *mt)
 {
     FileAsyncReader *This = impl_from_BasePin(iface);
-    if (iPosition < 0)
+    AsyncReader *filter = impl_from_IBaseFilter(This->pin.pin.pinInfo.pFilter);
+
+    if (index < 0)
         return E_INVALIDARG;
-    if (iPosition > 0)
+    else if (index > 1)
         return VFW_S_NO_MORE_ITEMS;
-    CopyMediaType(pmt, impl_from_IBaseFilter(This->pin.pin.pinInfo.pFilter)->pmt);
+
+    if (index == 0)
+        CopyMediaType(mt, filter->pmt);
+    else if (index == 1)
+        CopyMediaType(mt, &default_mt);
     return S_OK;
 }
 
@@ -855,9 +819,9 @@ static const IPinVtbl FileAsyncReaderPin_Vtbl =
 /* specific AM_MEDIA_TYPE - it cannot be NULL */
 /* this differs from standard OutputPin_AttemptConnection only in that it
  * doesn't need the IMemInputPin interface on the receiving pin */
-static HRESULT WINAPI FileAsyncReaderPin_AttemptConnection(BasePin * iface, IPin * pReceivePin, const AM_MEDIA_TYPE * pmt)
+static HRESULT WINAPI FileAsyncReaderPin_AttemptConnection(BaseOutputPin *This,
+        IPin *pReceivePin, const AM_MEDIA_TYPE *pmt)
 {
-    BaseOutputPin *This = impl_BaseOutputPin_from_BasePin(iface);
     HRESULT hr;
 
     TRACE("%p->(%p, %p)\n", This, pReceivePin, pmt);
@@ -869,7 +833,7 @@ static HRESULT WINAPI FileAsyncReaderPin_AttemptConnection(BasePin * iface, IPin
     IPin_AddRef(pReceivePin);
     CopyMediaType(&This->pin.mtCurrent, pmt);
 
-    hr = IPin_ReceiveConnection(pReceivePin, &iface->IPin_iface, pmt);
+    hr = IPin_ReceiveConnection(pReceivePin, &This->pin.IPin_iface, pmt);
 
     if (FAILED(hr))
     {
@@ -902,13 +866,11 @@ static HRESULT WINAPI FileAsyncReaderPin_DecideBufferSize(BaseOutputPin *iface,
 static const BaseOutputPinFuncTable output_BaseOutputFuncTable = {
     {
         FileAsyncReaderPin_CheckMediaType,
-        FileAsyncReaderPin_AttemptConnection,
-        BasePinImpl_GetMediaTypeVersion,
         FileAsyncReaderPin_GetMediaType
     },
+    FileAsyncReaderPin_AttemptConnection,
     FileAsyncReaderPin_DecideBufferSize,
     BaseOutputPinImpl_DecideAllocator,
-    BaseOutputPinImpl_BreakConnect
 };
 
 static HRESULT FileAsyncReader_Construct(HANDLE hFile, IBaseFilter * pBaseFilter, LPCRITICAL_SECTION pCritSec, IPin ** ppPin)
@@ -1214,7 +1176,6 @@ static HRESULT WINAPI FileAsyncReader_WaitForNext(IAsyncReader * iface, DWORD dw
     if (SUCCEEDED(hr))
     {
         REFERENCE_TIME rtStart, rtStop;
-        REFERENCE_TIME rtSampleStart, rtSampleStop;
         DATAREQUEST *pDataRq = This->sample_list + buffer;
         DWORD dwBytes = 0;
 
@@ -1239,16 +1200,7 @@ static HRESULT WINAPI FileAsyncReader_WaitForNext(IAsyncReader * iface, DWORD dw
         rtStart = MEDIATIME_FROM_BYTES(rtStart);
         rtStop = rtStart + MEDIATIME_FROM_BYTES(dwBytes);
 
-        IMediaSample_GetTime(pDataRq->pSample, &rtSampleStart, &rtSampleStop);
-        assert(rtStart == rtSampleStart);
-        assert(rtStop <= rtSampleStop);
-
         IMediaSample_SetTime(pDataRq->pSample, &rtStart, &rtStop);
-        assert(rtStart == rtSampleStart);
-        if (hr == S_OK)
-            assert(rtStop == rtSampleStop);
-        else
-            assert(rtStop == rtStart);
 
         This->sample_list[buffer].pSample = NULL;
         assert(This->oldest_sample < This->samples);
@@ -1275,64 +1227,81 @@ static HRESULT WINAPI FileAsyncReader_WaitForNext(IAsyncReader * iface, DWORD dw
     return hr;
 }
 
-static HRESULT WINAPI FileAsyncReader_SyncRead(IAsyncReader * iface, LONGLONG llPosition, LONG lLength, BYTE * pBuffer);
-
-static HRESULT WINAPI FileAsyncReader_SyncReadAligned(IAsyncReader * iface, IMediaSample * pSample)
+static BOOL sync_read(HANDLE file, LONGLONG offset, LONG length, BYTE *buffer, DWORD *read_len)
 {
-    BYTE * pBuffer;
-    REFERENCE_TIME tStart;
-    REFERENCE_TIME tStop;
-    HRESULT hr;
+    OVERLAPPED ovl = {0};
+    BOOL ret;
 
-    TRACE("(%p)\n", pSample);
+    ovl.hEvent = CreateEventW(NULL, TRUE, FALSE, NULL);
+    ovl.u.s.Offset = (DWORD)offset;
+    ovl.u.s.OffsetHigh = offset >> 32;
 
-    hr = IMediaSample_GetTime(pSample, &tStart, &tStop);
+    *read_len = 0;
 
-    if (SUCCEEDED(hr))
-        hr = IMediaSample_GetPointer(pSample, &pBuffer);
+    ret = ReadFile(file, buffer, length, NULL, &ovl);
+    if (ret || GetLastError() == ERROR_IO_PENDING)
+        ret = GetOverlappedResult(file, &ovl, read_len, TRUE);
 
-    if (SUCCEEDED(hr))
-        hr = FileAsyncReader_SyncRead(iface, 
-            BYTES_FROM_MEDIATIME(tStart),
-            (LONG) BYTES_FROM_MEDIATIME(tStop - tStart),
-            pBuffer);
+    TRACE("Returning %u bytes.\n", *read_len);
 
-    TRACE("-- %x\n", hr);
-    return hr;
+    CloseHandle(ovl.hEvent);
+    return ret;
 }
 
-static HRESULT WINAPI FileAsyncReader_SyncRead(IAsyncReader * iface, LONGLONG llPosition, LONG lLength, BYTE * pBuffer)
+static HRESULT WINAPI FileAsyncReader_SyncReadAligned(IAsyncReader *iface, IMediaSample *sample)
 {
-    OVERLAPPED ovl;
-    HRESULT hr = S_OK;
-    FileAsyncReader *This = impl_from_IAsyncReader(iface);
+    FileAsyncReader *filter = impl_from_IAsyncReader(iface);
+    REFERENCE_TIME start_time, end_time;
+    DWORD read_len;
+    BYTE *buffer;
+    LONG length;
+    HRESULT hr;
+    BOOL ret;
 
-    TRACE("%p->(%s, %d, %p)\n", This, wine_dbgstr_longlong(llPosition), lLength, pBuffer);
+    TRACE("filter %p, sample %p.\n", filter, sample);
 
-    ZeroMemory(&ovl, sizeof(ovl));
+    hr = IMediaSample_GetTime(sample, &start_time, &end_time);
 
-    ovl.hEvent = CreateEventW(NULL, 0, 0, NULL);
-    /* NOTE: llPosition is the actual byte position to start reading from */
-    ovl.u.s.Offset = (DWORD) llPosition;
-    ovl.u.s.OffsetHigh = (DWORD) (llPosition >> (sizeof(DWORD) * 8));
-
-    if (!ReadFile(This->hFile, pBuffer, lLength, NULL, &ovl))
-        hr = HRESULT_FROM_WIN32(GetLastError());
-
-    if (hr == HRESULT_FROM_WIN32(ERROR_IO_PENDING))
-        hr = S_OK;
+    if (SUCCEEDED(hr))
+        hr = IMediaSample_GetPointer(sample, &buffer);
 
     if (SUCCEEDED(hr))
     {
-        DWORD dwBytesRead;
-
-        if (!GetOverlappedResult(This->hFile, &ovl, &dwBytesRead, TRUE))
+        length = BYTES_FROM_MEDIATIME(end_time - start_time);
+        ret = sync_read(filter->hFile, BYTES_FROM_MEDIATIME(start_time), length, buffer, &read_len);
+        if (ret)
+            hr = (read_len == length) ? S_OK : S_FALSE;
+        else if (GetLastError() == ERROR_HANDLE_EOF)
+            hr = S_OK;
+        else
             hr = HRESULT_FROM_WIN32(GetLastError());
     }
 
-    CloseHandle(ovl.hEvent);
+    if (SUCCEEDED(hr))
+        IMediaSample_SetActualDataLength(sample, read_len);
+
+    return hr;
+}
+
+static HRESULT WINAPI FileAsyncReader_SyncRead(IAsyncReader *iface,
+        LONGLONG offset, LONG length, BYTE *buffer)
+{
+    FileAsyncReader *filter = impl_from_IAsyncReader(iface);
+    DWORD read_len;
+    HRESULT hr;
+    BOOL ret;
+
+    TRACE("filter %p, offset %s, length %d, buffer %p.\n",
+            filter, wine_dbgstr_longlong(offset), length, buffer);
+
+    ret = sync_read(filter->hFile, offset, length, buffer, &read_len);
+    if (ret)
+        hr = (read_len == length) ? S_OK : S_FALSE;
+    else if (GetLastError() == ERROR_HANDLE_EOF)
+        hr = S_FALSE;
+    else
+        hr = HRESULT_FROM_WIN32(GetLastError());
 
-    TRACE("-- %x\n", hr);
     return hr;
 }
 
