@@ -35,7 +35,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 
-static const WCHAR wcsInputPinName[] = {'i','n','p','u','t',' ','p','i','n',0};
 static const IMediaSeekingVtbl Parser_Seeking_Vtbl;
 static const IPinVtbl Parser_OutputPin_Vtbl;
 static const IPinVtbl Parser_InputPin_Vtbl;
@@ -47,7 +46,6 @@ static HRESULT WINAPI Parser_OutputPin_DecideBufferSize(BaseOutputPin *iface, IM
 static HRESULT WINAPI Parser_OutputPin_CheckMediaType(BasePin *pin, const AM_MEDIA_TYPE *pmt);
 static HRESULT WINAPI Parser_OutputPin_GetMediaType(BasePin *iface, int iPosition, AM_MEDIA_TYPE *pmt);
 static HRESULT WINAPI Parser_OutputPin_DecideAllocator(BaseOutputPin *This, IMemInputPin *pPin, IMemAllocator **pAlloc);
-static HRESULT WINAPI Parser_OutputPin_BreakConnect(BaseOutputPin *This);
 
 static inline ParserImpl *impl_from_IMediaSeeking( IMediaSeeking *iface )
 {
@@ -64,42 +62,29 @@ static inline ParserImpl *impl_from_BaseFilter( BaseFilter *iface )
     return CONTAINING_RECORD(iface, ParserImpl, filter);
 }
 
-/* FIXME: WRONG */
-static IPin* WINAPI Parser_GetPin(BaseFilter *iface, int pos)
+IPin *parser_get_pin(BaseFilter *iface, unsigned int index)
 {
-    ParserImpl *This = impl_from_BaseFilter(iface);
+    ParserImpl *filter = impl_from_BaseFilter(iface);
 
-    TRACE("%p->(%x)\n", This, pos);
-
-    /* Input pin also has a pin, hence the > and not >= */
-    if (pos > This->cStreams || pos < 0)
+    if (index > filter->cStreams)
         return NULL;
 
-    IPin_AddRef(This->ppPins[pos]);
-    return This->ppPins[pos];
+    IPin_AddRef(filter->ppPins[index]);
+    return filter->ppPins[index];
 }
 
-static LONG WINAPI Parser_GetPinCount(BaseFilter *iface)
-{
-    ParserImpl *This = impl_from_BaseFilter(iface);
-
-    TRACE("%p->()\n", This);
-
-    return This->cStreams;
-}
-
-static const BaseFilterFuncTable BaseFuncTable = {
-    Parser_GetPin,
-    Parser_GetPinCount
-};
-
-HRESULT Parser_Create(ParserImpl* pParser, const IBaseFilterVtbl *Parser_Vtbl, const CLSID* pClsid, PFN_PROCESS_SAMPLE fnProcessSample, PFN_QUERY_ACCEPT fnQueryAccept, PFN_PRE_CONNECT fnPreConnect, PFN_CLEANUP fnCleanup, PFN_DISCONNECT fnDisconnect, REQUESTPROC fnRequest, STOPPROCESSPROC fnDone, SourceSeeking_ChangeStop stop, SourceSeeking_ChangeStart start, SourceSeeking_ChangeRate rate)
+HRESULT Parser_Create(ParserImpl *pParser, const IBaseFilterVtbl *vtbl, IUnknown *outer,
+        const CLSID *clsid, const BaseFilterFuncTable *func_table, const WCHAR *sink_name,
+        PFN_PROCESS_SAMPLE fnProcessSample, PFN_QUERY_ACCEPT fnQueryAccept, PFN_PRE_CONNECT fnPreConnect,
+        PFN_CLEANUP fnCleanup, PFN_DISCONNECT fnDisconnect, REQUESTPROC fnRequest,
+        STOPPROCESSPROC fnDone, SourceSeeking_ChangeStop stop,
+        SourceSeeking_ChangeStart start, SourceSeeking_ChangeRate rate)
 {
     HRESULT hr;
     PIN_INFO piInput;
 
-    /* pTransformFilter is already allocated */
-    BaseFilter_Init(&pParser->filter, Parser_Vtbl, pClsid, (DWORD_PTR)(__FILE__ ": ParserImpl.csFilter"), &BaseFuncTable);
+    strmbase_filter_init(&pParser->filter, vtbl, outer, clsid,
+            (DWORD_PTR)(__FILE__ ": ParserImpl.csFilter"), func_table);
 
     pParser->fnDisconnect = fnDisconnect;
 
@@ -109,7 +94,7 @@ HRESULT Parser_Create(ParserImpl* pParser, const IBaseFilterVtbl *Parser_Vtbl, c
     /* construct input pin */
     piInput.dir = PINDIR_INPUT;
     piInput.pFilter = &pParser->filter.IBaseFilter_iface;
-    lstrcpynW(piInput.achName, wcsInputPinName, sizeof(piInput.achName) / sizeof(piInput.achName[0]));
+    lstrcpynW(piInput.achName, sink_name, ARRAY_SIZE(piInput.achName));
 
     if (!start)
         start = Parser_ChangeStart;
@@ -132,7 +117,7 @@ HRESULT Parser_Create(ParserImpl* pParser, const IBaseFilterVtbl *Parser_Vtbl, c
     else
     {
         CoTaskMemFree(pParser->ppPins);
-        BaseFilterImpl_Release(&pParser->filter.IBaseFilter_iface);
+        strmbase_filter_cleanup(&pParser->filter);
         CoTaskMemFree(pParser);
     }
 
@@ -175,7 +160,6 @@ void Parser_Destroy(ParserImpl *This)
     ULONG pinref;
     HRESULT hr;
 
-    assert(!This->filter.refCount);
     PullPin_WaitForStateChange(This->pInputPin, INFINITE);
 
     /* Don't need to clean up output pins, freeing input pin will do that */
@@ -200,38 +184,12 @@ void Parser_Destroy(ParserImpl *This)
     }
 
     CoTaskMemFree(This->ppPins);
-    BaseFilter_Destroy(&This->filter);
+    strmbase_filter_cleanup(&This->filter);
 
     TRACE("Destroying parser\n");
     CoTaskMemFree(This);
 }
 
-ULONG WINAPI Parser_Release(IBaseFilter * iface)
-{
-    ParserImpl *This = impl_from_IBaseFilter(iface);
-    ULONG refCount = InterlockedDecrement(&This->filter.refCount);
-
-    TRACE("(%p)->() Release from %d\n", This, refCount + 1);
-
-    if (!refCount)
-        Parser_Destroy(This);
-
-    return refCount;
-}
-
-/** IPersist methods **/
-
-HRESULT WINAPI Parser_GetClassID(IBaseFilter * iface, CLSID * pClsid)
-{
-    ParserImpl *This = impl_from_IBaseFilter(iface);
-
-    TRACE("%p->(%p)\n", This, pClsid);
-
-    *pClsid = This->filter.clsid;
-
-    return S_OK;
-}
-
 /** IMediaFilter methods **/
 
 HRESULT WINAPI Parser_Stop(IBaseFilter * iface)
@@ -390,43 +348,14 @@ HRESULT WINAPI Parser_SetSyncSource(IBaseFilter * iface, IReferenceClock *pClock
     return S_OK;
 }
 
-HRESULT WINAPI Parser_GetSyncSource(IBaseFilter * iface, IReferenceClock **ppClock)
-{
-    return BaseFilterImpl_GetSyncSource(iface, ppClock);
-}
-
-/** IBaseFilter implementation **/
-
-HRESULT WINAPI Parser_EnumPins(IBaseFilter * iface, IEnumPins **ppEnum)
-{
-    return BaseFilterImpl_EnumPins(iface,ppEnum);
-}
-
-HRESULT WINAPI Parser_QueryFilterInfo(IBaseFilter * iface, FILTER_INFO *pInfo)
-{
-    return BaseFilterImpl_QueryFilterInfo(iface, pInfo);
-}
-
-HRESULT WINAPI Parser_JoinFilterGraph(IBaseFilter * iface, IFilterGraph *pGraph, LPCWSTR pName)
-{
-    return BaseFilterImpl_JoinFilterGraph(iface, pGraph, pName);
-}
-
-HRESULT WINAPI Parser_QueryVendorInfo(IBaseFilter * iface, LPWSTR *pVendorInfo)
-{
-    return BaseFilterImpl_QueryVendorInfo(iface, pVendorInfo);
-}
-
 static const BaseOutputPinFuncTable output_BaseOutputFuncTable = {
     {
         Parser_OutputPin_CheckMediaType,
-        BaseOutputPinImpl_AttemptConnection,
-        BasePinImpl_GetMediaTypeVersion,
         Parser_OutputPin_GetMediaType
     },
+    BaseOutputPinImpl_AttemptConnection,
     Parser_OutputPin_DecideBufferSize,
     Parser_OutputPin_DecideAllocator,
-    Parser_OutputPin_BreakConnect
 };
 
 HRESULT Parser_AddPin(ParserImpl * This, const PIN_INFO * piOutput, ALLOCATOR_PROPERTIES * props, const AM_MEDIA_TYPE * amt)
@@ -465,6 +394,22 @@ HRESULT Parser_AddPin(ParserImpl * This, const PIN_INFO * piOutput, ALLOCATOR_PR
     return hr;
 }
 
+static HRESULT WINAPI break_connection(IPin *iface)
+{
+    Parser_OutputPin *pin = unsafe_impl_Parser_OutputPin_from_IPin(iface);
+    HRESULT hr;
+
+    if (!pin->pin.pin.pConnectedTo || !pin->pin.pMemInputPin)
+        hr = VFW_E_NOT_CONNECTED;
+    else
+    {
+        hr = IPin_Disconnect(pin->pin.pin.pConnectedTo);
+        IPin_Disconnect(iface);
+    }
+
+    return hr;
+}
+
 static HRESULT Parser_RemoveOutputPins(ParserImpl * This)
 {
     /* NOTE: should be in critical section when calling this function */
@@ -480,7 +425,7 @@ static HRESULT Parser_RemoveOutputPins(ParserImpl * This)
 
     for (i = 0; i < This->cStreams; i++)
     {
-        hr = ((BaseOutputPin *)ppOldPins[i + 1])->pFuncsTable->pfnBreakConnect((BaseOutputPin *)ppOldPins[i + 1]);
+        hr = break_connection(ppOldPins[i + 1]);
         TRACE("Disconnect: %08x\n", hr);
         IPin_Release(ppOldPins[i + 1]);
     }
@@ -606,26 +551,6 @@ static HRESULT WINAPI Parser_OutputPin_DecideAllocator(BaseOutputPin *iface, IMe
     return hr;
 }
 
-static HRESULT WINAPI Parser_OutputPin_BreakConnect(BaseOutputPin *This)
-{
-    HRESULT hr;
-
-    TRACE("(%p)->()\n", This);
-
-    EnterCriticalSection(This->pin.pCritSec);
-    if (!This->pin.pConnectedTo || !This->pMemInputPin)
-        hr = VFW_E_NOT_CONNECTED;
-    else
-    {
-        hr = IPin_Disconnect(This->pin.pConnectedTo);
-        IPin_Disconnect(&This->pin.IPin_iface);
-    }
-    LeaveCriticalSection(This->pin.pCritSec);
-
-    return hr;
-}
-
-
 static HRESULT WINAPI Parser_OutputPin_QueryInterface(IPin * iface, REFIID riid, LPVOID * ppv)
 {
     Parser_OutputPin *This = unsafe_impl_Parser_OutputPin_from_IPin(iface);
