@@ -31,6 +31,7 @@
 #include "winternl.h"
 #include "winreg.h"
 #include "setupapi.h"
+#include "cfgmgr32.h"
 #include "winioctl.h"
 #include "ddk/wdm.h"
 #include "ddk/hidport.h"
@@ -43,6 +44,8 @@
 WINE_DEFAULT_DEBUG_CHANNEL(plugplay);
 WINE_DECLARE_DEBUG_CHANNEL(hid_report);
 
+static const WCHAR backslashW[] = {'\\',0};
+
 #define VID_MICROSOFT 0x045e
 
 static const WORD PID_XBOX_CONTROLLERS[] =  {
@@ -53,12 +56,18 @@ static const WORD PID_XBOX_CONTROLLERS[] =  {
     0x028f, /* Xbox360 Wireless Controller */
     0x02d1, /* Xbox One Controller */
     0x02dd, /* Xbox One Controller (Covert Forces/Firmware 2015) */
+    0x02e0, /* Xbox One X Controller */
     0x02e3, /* Xbox One Elite Controller */
     0x02e6, /* Wireless XBox Controller Dongle */
     0x02ea, /* Xbox One S Controller */
+    0x02fd, /* Xbox One S Controller (Firmware 2017) */
     0x0719, /* Xbox 360 Wireless Adapter */
 };
 
+static DRIVER_OBJECT *driver_obj;
+
+HANDLE driver_key;
+
 struct pnp_device
 {
     struct list entry;
@@ -69,7 +78,7 @@ struct device_extension
 {
     struct pnp_device *pnp_device;
 
-    WORD vid, pid;
+    WORD vid, pid, input;
     DWORD uid, version, index;
     BOOL is_gamepad;
     WCHAR *serial;
@@ -99,7 +108,7 @@ static CRITICAL_SECTION device_list_cs = { &critsect_debug, -1, 0, 0, 0, 0 };
 static struct list pnp_devset = LIST_INIT(pnp_devset);
 
 static const WCHAR zero_serialW[]= {'0','0','0','0',0};
-static const WCHAR imW[] = {'I','M',0};
+static const WCHAR miW[] = {'M','I',0};
 static const WCHAR igW[] = {'I','G',0};
 
 static inline WCHAR *strdupW(const WCHAR *src)
@@ -117,15 +126,15 @@ void *get_platform_private(DEVICE_OBJECT *device)
     return ext->platform_private;
 }
 
-static DWORD get_vidpid_index(WORD vid, WORD pid)
+static DWORD get_device_index(WORD vid, WORD pid, WORD input)
 {
     struct pnp_device *ptr;
-    DWORD index = 1;
+    DWORD index = 0;
 
     LIST_FOR_EACH_ENTRY(ptr, &pnp_devset, struct pnp_device, entry)
     {
         struct device_extension *ext = (struct device_extension *)ptr->device->DeviceExtension;
-        if (ext->vid == vid && ext->pid == pid)
+        if (ext->vid == vid && ext->pid == pid && ext->input == input)
             index = max(ext->index + 1, index);
     }
 
@@ -134,29 +143,40 @@ static DWORD get_vidpid_index(WORD vid, WORD pid)
 
 static WCHAR *get_instance_id(DEVICE_OBJECT *device)
 {
-    static const WCHAR formatW[] =  {'%','s','\\','V','i','d','_','%','0','4','x','&', 'P','i','d','_','%','0','4','x','&',
-                                     '%','s','_','%','i','\\','%','i','&','%','s','&','%','x',0};
+    static const WCHAR formatW[] =  {'%','i','&','%','s','&','%','x','&','%','i',0};
     struct device_extension *ext = (struct device_extension *)device->DeviceExtension;
     const WCHAR *serial = ext->serial ? ext->serial : zero_serialW;
-    DWORD len = strlenW(ext->busid) + strlenW(serial) + 64;
+    DWORD len = strlenW(serial) + 33;
     WCHAR *dst;
 
-    if ((dst = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR))))
-        sprintfW(dst, formatW, ext->busid, ext->vid, ext->pid, ext->is_gamepad ? igW : imW,
-                 ext->index, ext->version, serial, ext->uid);
+    if ((dst = ExAllocatePool(PagedPool, len * sizeof(WCHAR))))
+        sprintfW(dst, formatW, ext->version, serial, ext->uid, ext->index);
 
     return dst;
 }
 
 static WCHAR *get_device_id(DEVICE_OBJECT *device)
 {
-    static const WCHAR formatW[] =  {'%','s','\\','V','i','d','_','%','0','4','x','&','P','i','d','_','%','0','4','x',0};
+    static const WCHAR formatW[] = {'%','s','\\','v','i','d','_','%','0','4','x',
+            '&','p','i','d','_','%','0','4','x',0};
+    static const WCHAR format_inputW[] = {'%','s','\\','v','i','d','_','%','0','4','x',
+            '&','p','i','d','_','%','0','4','x','&','%','s','_','%','i',0};
     struct device_extension *ext = (struct device_extension *)device->DeviceExtension;
-    DWORD len = strlenW(ext->busid) + 19;
+    DWORD len = strlenW(ext->busid) + 34;
     WCHAR *dst;
 
-    if ((dst = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR))))
-        sprintfW(dst, formatW, ext->busid, ext->vid, ext->pid);
+    if ((dst = ExAllocatePool(PagedPool, len * sizeof(WCHAR))))
+    {
+        if (ext->input == (WORD)-1)
+        {
+            sprintfW(dst, formatW, ext->busid, ext->vid, ext->pid);
+        }
+        else
+        {
+            sprintfW(dst, format_inputW, ext->busid, ext->vid, ext->pid,
+                    ext->is_gamepad ? igW : miW, ext->input);
+        }
+    }
 
     return dst;
 }
@@ -164,52 +184,36 @@ static WCHAR *get_device_id(DEVICE_OBJECT *device)
 static WCHAR *get_compatible_ids(DEVICE_OBJECT *device)
 {
     struct device_extension *ext = (struct device_extension *)device->DeviceExtension;
-    WCHAR *iid, *did, *dst, *ptr;
-    DWORD len;
+    WCHAR *dst;
 
-    if (!(iid = get_instance_id(device)))
-        return NULL;
-
-    if (!(did = get_device_id(device)))
+    if ((dst = ExAllocatePool(PagedPool, (strlenW(ext->busid) + 2) * sizeof(WCHAR))))
     {
-        HeapFree(GetProcessHeap(), 0, iid);
-        return NULL;
+        strcpyW(dst, ext->busid);
+        dst[strlenW(dst) + 1] = 0;
     }
 
-    len = strlenW(iid) + strlenW(did) + strlenW(ext->busid) + 4;
-    if ((dst = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR))))
-    {
-        ptr = dst;
-        strcpyW(ptr, iid);
-        ptr += strlenW(iid) + 1;
-        strcpyW(ptr, did);
-        ptr += strlenW(did) + 1;
-        strcpyW(ptr, ext->busid);
-        ptr += strlenW(ext->busid) + 1;
-        *ptr = 0;
-    }
-
-    HeapFree(GetProcessHeap(), 0, iid);
-    HeapFree(GetProcessHeap(), 0, did);
     return dst;
 }
 
-DEVICE_OBJECT *bus_create_hid_device(DRIVER_OBJECT *driver, const WCHAR *busidW, WORD vid, WORD pid,
-                                     DWORD version, DWORD uid, const WCHAR *serialW, BOOL is_gamepad,
+DEVICE_OBJECT *bus_create_hid_device(const WCHAR *busidW, WORD vid, WORD pid,
+                                     WORD input, DWORD version, DWORD uid, const WCHAR *serialW, BOOL is_gamepad,
                                      const GUID *class, const platform_vtbl *vtbl, DWORD platform_data_size)
 {
     static const WCHAR device_name_fmtW[] = {'\\','D','e','v','i','c','e','\\','%','s','#','%','p',0};
+    WCHAR *id, instance[MAX_DEVICE_ID_LEN];
     struct device_extension *ext;
     struct pnp_device *pnp_dev;
     DEVICE_OBJECT *device;
     UNICODE_STRING nameW;
     WCHAR dev_name[256];
     HDEVINFO devinfo;
+    SP_DEVINFO_DATA data = {sizeof(data)};
     NTSTATUS status;
     DWORD length;
 
-    TRACE("(%p, %s, %04x, %04x, %u, %u, %s, %u, %s, %p, %u)\n", driver, debugstr_w(busidW), vid, pid,
-          version, uid, debugstr_w(serialW), is_gamepad, debugstr_guid(class), vtbl, platform_data_size);
+    TRACE("(%s, %04x, %04x, %04x, %u, %u, %s, %u, %s, %p, %u)\n",
+            debugstr_w(busidW), vid, pid, input, version, uid, debugstr_w(serialW),
+            is_gamepad, debugstr_guid(class), vtbl, platform_data_size);
 
     if (!(pnp_dev = HeapAlloc(GetProcessHeap(), 0, sizeof(*pnp_dev))))
         return NULL;
@@ -217,7 +221,7 @@ DEVICE_OBJECT *bus_create_hid_device(DRIVER_OBJECT *driver, const WCHAR *busidW,
     sprintfW(dev_name, device_name_fmtW, busidW, pnp_dev);
     RtlInitUnicodeString(&nameW, dev_name);
     length = FIELD_OFFSET(struct device_extension, platform_private[platform_data_size]);
-    status = IoCreateDevice(driver, length, &nameW, 0, 0, FALSE, &device);
+    status = IoCreateDevice(driver_obj, length, &nameW, 0, 0, FALSE, &device);
     if (status)
     {
         FIXME("failed to create device error %x\n", status);
@@ -232,9 +236,10 @@ DEVICE_OBJECT *bus_create_hid_device(DRIVER_OBJECT *driver, const WCHAR *busidW,
     ext->pnp_device         = pnp_dev;
     ext->vid                = vid;
     ext->pid                = pid;
+    ext->input              = input;
     ext->uid                = uid;
     ext->version            = version;
-    ext->index              = get_vidpid_index(vid, pid);
+    ext->index              = get_device_index(vid, pid, input);
     ext->is_gamepad         = is_gamepad;
     ext->serial             = strdupW(serialW);
     ext->busid              = busidW;
@@ -256,26 +261,40 @@ DEVICE_OBJECT *bus_create_hid_device(DRIVER_OBJECT *driver, const WCHAR *busidW,
 
     LeaveCriticalSection(&device_list_cs);
 
-    devinfo = SetupDiGetClassDevsW(class, NULL, NULL, DIGCF_DEVICEINTERFACE);
-    if (devinfo)
+    devinfo = SetupDiCreateDeviceInfoList(class, NULL);
+    if (devinfo == INVALID_HANDLE_VALUE)
     {
-        SP_DEVINFO_DATA data;
-        WCHAR *instance;
-
-        data.cbSize = sizeof(data);
-        if (!(instance = get_instance_id(device)))
-            ERR("failed to generate instance id\n");
-        else if (!SetupDiCreateDeviceInfoW(devinfo, instance, class, NULL, NULL, DICD_INHERIT_CLASSDRVS, &data))
-            ERR("failed to create device info: %x\n", GetLastError());
-        else if (!SetupDiRegisterDeviceInfo(devinfo, &data, 0, NULL, NULL, NULL))
-            ERR("failed to register device info: %x\n", GetLastError());
-
-        HeapFree(GetProcessHeap(), 0, instance);
-        SetupDiDestroyDeviceInfoList(devinfo);
+        ERR("failed to create device info list, error %#x\n", GetLastError());
+        goto error;
     }
-    else
-        ERR("failed to get ClassDevs: %x\n", GetLastError());
 
+    if (!(id = get_device_id(device)))
+    {
+        ERR("failed to generate instance id\n");
+        goto error;
+    }
+    strcpyW(instance, id);
+    ExFreePool(id);
+
+    if (!(id = get_instance_id(device)))
+    {
+        ERR("failed to generate instance id\n");
+        goto error;
+    }
+    strcatW(instance, backslashW);
+    strcatW(instance, id);
+    ExFreePool(id);
+
+    if (SetupDiCreateDeviceInfoW(devinfo, instance, class, NULL, NULL, DICD_INHERIT_CLASSDRVS, &data))
+    {
+        if (!SetupDiRegisterDeviceInfo(devinfo, &data, 0, NULL, NULL, NULL))
+            ERR("failed to register device info, error %#x\n", GetLastError());
+    }
+    else if (GetLastError() != ERROR_DEVINST_ALREADY_EXISTS)
+        ERR("failed to create device info, error %#x\n", GetLastError());
+
+error:
+    SetupDiDestroyDeviceInfoList(devinfo);
     return device;
 }
 
@@ -360,6 +379,56 @@ void bus_remove_hid_device(DEVICE_OBJECT *device)
     HeapFree(GetProcessHeap(), 0, pnp_device);
 }
 
+static NTSTATUS build_device_relations(DEVICE_RELATIONS **devices)
+{
+    int i;
+    struct pnp_device *ptr;
+
+    EnterCriticalSection(&device_list_cs);
+    *devices = ExAllocatePool(PagedPool, offsetof(DEVICE_RELATIONS, Objects[list_count(&pnp_devset)]));
+
+    if (!*devices)
+    {
+        LeaveCriticalSection(&device_list_cs);
+        return STATUS_INSUFFICIENT_RESOURCES;
+    }
+
+    i = 0;
+    LIST_FOR_EACH_ENTRY(ptr, &pnp_devset, struct pnp_device, entry)
+    {
+        (*devices)->Objects[i] = ptr->device;
+        i++;
+    }
+    LeaveCriticalSection(&device_list_cs);
+    (*devices)->Count = i;
+    return STATUS_SUCCESS;
+}
+
+static NTSTATUS handle_IRP_MN_QUERY_DEVICE_RELATIONS(IRP *irp)
+{
+    NTSTATUS status = irp->IoStatus.u.Status;
+    IO_STACK_LOCATION *irpsp = IoGetCurrentIrpStackLocation( irp );
+
+    TRACE("IRP_MN_QUERY_DEVICE_RELATIONS\n");
+    switch (irpsp->Parameters.QueryDeviceRelations.Type)
+    {
+        case EjectionRelations:
+        case RemovalRelations:
+        case TargetDeviceRelation:
+        case PowerRelations:
+            FIXME("Unhandled Device Relation %x\n",irpsp->Parameters.QueryDeviceRelations.Type);
+            break;
+        case BusRelations:
+            status = build_device_relations((DEVICE_RELATIONS**)&irp->IoStatus.Information);
+            break;
+        default:
+            FIXME("Unknown Device Relation %x\n",irpsp->Parameters.QueryDeviceRelations.Type);
+            break;
+    }
+
+    return status;
+}
+
 static NTSTATUS handle_IRP_MN_QUERY_ID(DEVICE_OBJECT *device, IRP *irp)
 {
     NTSTATUS status = irp->IoStatus.u.Status;
@@ -395,7 +464,7 @@ static NTSTATUS handle_IRP_MN_QUERY_ID(DEVICE_OBJECT *device, IRP *irp)
     return status;
 }
 
-NTSTATUS WINAPI common_pnp_dispatch(DEVICE_OBJECT *device, IRP *irp)
+static NTSTATUS WINAPI common_pnp_dispatch(DEVICE_OBJECT *device, IRP *irp)
 {
     NTSTATUS status = irp->IoStatus.u.Status;
     IO_STACK_LOCATION *irpsp = IoGetCurrentIrpStackLocation(irp);
@@ -404,6 +473,8 @@ NTSTATUS WINAPI common_pnp_dispatch(DEVICE_OBJECT *device, IRP *irp)
     {
         case IRP_MN_QUERY_DEVICE_RELATIONS:
             TRACE("IRP_MN_QUERY_DEVICE_RELATIONS\n");
+            status = handle_IRP_MN_QUERY_DEVICE_RELATIONS(irp);
+            irp->IoStatus.u.Status = status;
             break;
         case IRP_MN_QUERY_ID:
             TRACE("IRP_MN_QUERY_ID\n");
@@ -438,7 +509,7 @@ static NTSTATUS deliver_last_report(struct device_extension *ext, DWORD buffer_l
     }
 }
 
-NTSTATUS WINAPI hid_internal_dispatch(DEVICE_OBJECT *device, IRP *irp)
+static NTSTATUS WINAPI hid_internal_dispatch(DEVICE_OBJECT *device, IRP *irp)
 {
     NTSTATUS status = irp->IoStatus.u.Status;
     IO_STACK_LOCATION *irpsp = IoGetCurrentIrpStackLocation(irp);
@@ -641,9 +712,6 @@ void process_hid_report(DEVICE_OBJECT *device, BYTE *report, DWORD length)
             ext->buffer_size = length;
     }
 
-    if (!ext->last_report_read)
-        ERR_(hid_report)("Device reports coming in too fast, last report not read yet!\n");
-
     memcpy(ext->last_report, report, length);
     ext->last_report_size = length;
     ext->last_report_read = FALSE;
@@ -663,30 +731,19 @@ void process_hid_report(DEVICE_OBJECT *device, BYTE *report, DWORD length)
     LeaveCriticalSection(&ext->report_cs);
 }
 
-DWORD check_bus_option(UNICODE_STRING *registry_path, const UNICODE_STRING *option, DWORD default_value)
+DWORD check_bus_option(const UNICODE_STRING *option, DWORD default_value)
 {
-    OBJECT_ATTRIBUTES attr;
-    HANDLE key;
-    DWORD output = default_value;
+    char buffer[FIELD_OFFSET(KEY_VALUE_PARTIAL_INFORMATION, Data[sizeof(DWORD)])];
+    KEY_VALUE_PARTIAL_INFORMATION *info = (KEY_VALUE_PARTIAL_INFORMATION*)buffer;
+    DWORD size;
 
-    InitializeObjectAttributes(&attr, registry_path, OBJ_CASE_INSENSITIVE | OBJ_KERNEL_HANDLE, NULL, NULL);
-    if (NtOpenKey(&key, KEY_ALL_ACCESS, &attr) == STATUS_SUCCESS)
+    if (NtQueryValueKey(driver_key, option, KeyValuePartialInformation, info, sizeof(buffer), &size) == STATUS_SUCCESS)
     {
-        DWORD size;
-        char buffer[FIELD_OFFSET(KEY_VALUE_PARTIAL_INFORMATION, Data[sizeof(DWORD)])];
-
-        KEY_VALUE_PARTIAL_INFORMATION *info = (KEY_VALUE_PARTIAL_INFORMATION*)buffer;
-
-        if (NtQueryValueKey(key, option, KeyValuePartialInformation, info, sizeof(buffer), &size) == STATUS_SUCCESS)
-        {
-            if (info->Type == REG_DWORD)
-                output = *(DWORD*)info->Data;
-        }
-
-        NtClose(key);
+        if (info->Type == REG_DWORD)
+            return *(DWORD*)info->Data;
     }
 
-    return output;
+    return default_value;
 }
 
 BOOL is_xbox_gamepad(WORD vid, WORD pid)
@@ -696,32 +753,48 @@ BOOL is_xbox_gamepad(WORD vid, WORD pid)
     if (vid != VID_MICROSOFT)
         return FALSE;
 
-    for (i = 0; i < sizeof(PID_XBOX_CONTROLLERS)/sizeof(*PID_XBOX_CONTROLLERS); i++)
+    for (i = 0; i < ARRAY_SIZE(PID_XBOX_CONTROLLERS); i++)
         if (pid == PID_XBOX_CONTROLLERS[i]) return TRUE;
 
     return FALSE;
 }
 
+static void WINAPI driver_unload(DRIVER_OBJECT *driver)
+{
+    udev_driver_unload();
+    iohid_driver_unload();
+    sdl_driver_unload();
+    NtClose(driver_key);
+}
+
 NTSTATUS WINAPI DriverEntry( DRIVER_OBJECT *driver, UNICODE_STRING *path )
 {
-    static const WCHAR udevW[] = {'\\','D','r','i','v','e','r','\\','U','D','E','V',0};
-    static UNICODE_STRING udev = {sizeof(udevW) - sizeof(WCHAR), sizeof(udevW), (WCHAR *)udevW};
-    static const WCHAR iohidW[] = {'\\','D','r','i','v','e','r','\\','I','O','H','I','D',0};
-    static UNICODE_STRING iohid = {sizeof(iohidW) - sizeof(WCHAR), sizeof(iohidW), (WCHAR *)iohidW};
-    static const WCHAR sdlW[] = {'\\','D','r','i','v','e','r','\\','S','D','L','J','O','Y',0};
-    static UNICODE_STRING sdl = {sizeof(sdlW) - sizeof(WCHAR), sizeof(sdlW), (WCHAR *)sdlW};
     static const WCHAR SDL_enabledW[] = {'E','n','a','b','l','e',' ','S','D','L',0};
     static const UNICODE_STRING SDL_enabled = {sizeof(SDL_enabledW) - sizeof(WCHAR), sizeof(SDL_enabledW), (WCHAR*)SDL_enabledW};
+    OBJECT_ATTRIBUTES attr = {0};
+    NTSTATUS ret;
 
     TRACE( "(%p, %s)\n", driver, debugstr_w(path->Buffer) );
 
-    if (check_bus_option(path, &SDL_enabled, 1))
+    attr.Length = sizeof(attr);
+    attr.ObjectName = path;
+    attr.Attributes = OBJ_CASE_INSENSITIVE | OBJ_KERNEL_HANDLE;
+    if ((ret = NtOpenKey(&driver_key, KEY_ALL_ACCESS, &attr)) != STATUS_SUCCESS)
+        ERR("Failed to open driver key, status %#x.\n", ret);
+
+    driver_obj = driver;
+
+    driver->MajorFunction[IRP_MJ_PNP] = common_pnp_dispatch;
+    driver->MajorFunction[IRP_MJ_INTERNAL_DEVICE_CONTROL] = hid_internal_dispatch;
+    driver->DriverUnload = driver_unload;
+
+    if (check_bus_option(&SDL_enabled, 1))
     {
-        if (IoCreateDriver(&sdl, sdl_driver_init) == STATUS_SUCCESS)
+        if (sdl_driver_init() == STATUS_SUCCESS)
             return STATUS_SUCCESS;
     }
-    IoCreateDriver(&udev, udev_driver_init);
-    IoCreateDriver(&iohid, iohid_driver_init);
+    udev_driver_init();
+    iohid_driver_init();
 
     return STATUS_SUCCESS;
 }
