@@ -24,6 +24,23 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d);
 
+struct wined3d_device_vk
+{
+    struct wined3d_device d;
+
+    struct wined3d_context context_vk;
+
+    VkDevice vk_device;
+    VkQueue vk_queue;
+
+    struct wined3d_vk_info vk_info;
+};
+
+static inline struct wined3d_device_vk *wined3d_device_vk(struct wined3d_device *device)
+{
+    return CONTAINING_RECORD(device, struct wined3d_device_vk, d);
+}
+
 static inline const struct wined3d_adapter_vk *wined3d_adapter_vk_const(const struct wined3d_adapter *adapter)
 {
     return CONTAINING_RECORD(adapter, struct wined3d_adapter_vk, a);
@@ -35,6 +52,30 @@ static const char *debug_vk_version(uint32_t version)
             VK_VERSION_MAJOR(version), VK_VERSION_MINOR(version), VK_VERSION_PATCH(version));
 }
 
+static HRESULT hresult_from_vk_result(VkResult vr)
+{
+    switch (vr)
+    {
+        case VK_SUCCESS:
+            return S_OK;
+        case VK_ERROR_OUT_OF_HOST_MEMORY:
+            WARN("Out of host memory.\n");
+            return E_OUTOFMEMORY;
+        case VK_ERROR_OUT_OF_DEVICE_MEMORY:
+            WARN("Out of device memory.\n");
+            return E_OUTOFMEMORY;
+        case VK_ERROR_DEVICE_LOST:
+            WARN("Device lost.\n");
+            return E_FAIL;
+        case VK_ERROR_EXTENSION_NOT_PRESENT:
+            WARN("Extension not present.\n");
+            return E_FAIL;
+        default:
+            FIXME("Unhandled VkResult %d.\n", vr);
+            return E_FAIL;
+    }
+}
+
 #ifdef USE_WIN32_VULKAN
 static BOOL wined3d_load_vulkan(struct wined3d_vk_info *vk_info)
 {
@@ -96,10 +137,175 @@ static void adapter_vk_destroy(struct wined3d_adapter *adapter)
     heap_free(adapter_vk);
 }
 
-static BOOL adapter_vk_create_context(struct wined3d_context *context,
-        struct wined3d_texture *target, const struct wined3d_format *ds_format)
+static HRESULT wined3d_select_vulkan_queue_family(const struct wined3d_adapter_vk *adapter_vk,
+        uint32_t *queue_family_index)
 {
-    return TRUE;
+    VkPhysicalDevice physical_device = adapter_vk->physical_device;
+    const struct wined3d_vk_info *vk_info = &adapter_vk->vk_info;
+    VkQueueFamilyProperties *queue_properties;
+    uint32_t count, i;
+
+    VK_CALL(vkGetPhysicalDeviceQueueFamilyProperties(physical_device, &count, NULL));
+
+    if (!(queue_properties = heap_calloc(count, sizeof(*queue_properties))))
+        return E_OUTOFMEMORY;
+
+    VK_CALL(vkGetPhysicalDeviceQueueFamilyProperties(physical_device, &count, queue_properties));
+
+    for (i = 0; i < count; ++i)
+    {
+        if (queue_properties[i].queueFlags & VK_QUEUE_GRAPHICS_BIT)
+        {
+            *queue_family_index = i;
+            heap_free(queue_properties);
+            return WINED3D_OK;
+        }
+    }
+    heap_free(queue_properties);
+
+    WARN("Failed to find graphics queue.\n");
+    return E_FAIL;
+}
+
+static void wined3d_disable_vulkan_features(VkPhysicalDeviceFeatures *features)
+{
+    features->depthBounds = VK_FALSE;
+    features->alphaToOne = VK_FALSE;
+    features->textureCompressionETC2 = VK_FALSE;
+    features->textureCompressionASTC_LDR = VK_FALSE;
+    features->shaderStorageImageMultisample = VK_FALSE;
+    features->shaderUniformBufferArrayDynamicIndexing = VK_FALSE;
+    features->shaderSampledImageArrayDynamicIndexing = VK_FALSE;
+    features->shaderStorageBufferArrayDynamicIndexing = VK_FALSE;
+    features->shaderStorageImageArrayDynamicIndexing = VK_FALSE;
+    features->shaderInt16 = VK_FALSE;
+    features->shaderResourceResidency = VK_FALSE;
+    features->shaderResourceMinLod = VK_FALSE;
+    features->sparseBinding = VK_FALSE;
+    features->sparseResidencyBuffer = VK_FALSE;
+    features->sparseResidencyImage2D = VK_FALSE;
+    features->sparseResidencyImage3D = VK_FALSE;
+    features->sparseResidency2Samples = VK_FALSE;
+    features->sparseResidency4Samples = VK_FALSE;
+    features->sparseResidency8Samples = VK_FALSE;
+    features->sparseResidency16Samples = VK_FALSE;
+    features->sparseResidencyAliased = VK_FALSE;
+    features->inheritedQueries = VK_FALSE;
+}
+
+static HRESULT adapter_vk_create_device(struct wined3d *wined3d, const struct wined3d_adapter *adapter,
+        enum wined3d_device_type device_type, HWND focus_window, unsigned int flags, BYTE surface_alignment,
+        const enum wined3d_feature_level *levels, unsigned int level_count,
+        struct wined3d_device_parent *device_parent, struct wined3d_device **device)
+{
+    const struct wined3d_adapter_vk *adapter_vk = wined3d_adapter_vk_const(adapter);
+    const struct wined3d_vk_info *vk_info = &adapter_vk->vk_info;
+    static const float priorities[] = {1.0f};
+    struct wined3d_device_vk *device_vk;
+    VkDevice vk_device = VK_NULL_HANDLE;
+    VkDeviceQueueCreateInfo queue_info;
+    VkPhysicalDeviceFeatures features;
+    VkPhysicalDevice physical_device;
+    VkDeviceCreateInfo device_info;
+    uint32_t queue_family_index;
+    VkResult vr;
+    HRESULT hr;
+
+    if (!(device_vk = heap_alloc_zero(sizeof(*device_vk))))
+        return E_OUTOFMEMORY;
+
+    if (FAILED(hr = wined3d_select_vulkan_queue_family(adapter_vk, &queue_family_index)))
+        goto fail;
+
+    physical_device = adapter_vk->physical_device;
+
+    VK_CALL(vkGetPhysicalDeviceFeatures(physical_device, &features));
+    wined3d_disable_vulkan_features(&features);
+
+    queue_info.sType = VK_STRUCTURE_TYPE_DEVICE_QUEUE_CREATE_INFO;
+    queue_info.pNext = NULL;
+    queue_info.flags = 0;
+    queue_info.queueFamilyIndex = queue_family_index;
+    queue_info.queueCount = ARRAY_SIZE(priorities);
+    queue_info.pQueuePriorities = priorities;
+
+    device_info.sType = VK_STRUCTURE_TYPE_DEVICE_CREATE_INFO;
+    device_info.pNext = NULL;
+    device_info.flags = 0;
+    device_info.queueCreateInfoCount = 1;
+    device_info.pQueueCreateInfos = &queue_info;
+    device_info.enabledLayerCount = 0;
+    device_info.ppEnabledLayerNames = NULL;
+    device_info.enabledExtensionCount = 0;
+    device_info.ppEnabledExtensionNames = NULL;
+    device_info.pEnabledFeatures = &features;
+
+    if ((vr = VK_CALL(vkCreateDevice(physical_device, &device_info, NULL, &vk_device))) < 0)
+    {
+        WARN("Failed to create Vulkan device, vr %s.\n", wined3d_debug_vkresult(vr));
+        vk_device = VK_NULL_HANDLE;
+        hr = hresult_from_vk_result(vr);
+        goto fail;
+    }
+
+    device_vk->vk_device = vk_device;
+    VK_CALL(vkGetDeviceQueue(vk_device, queue_family_index, 0, &device_vk->vk_queue));
+
+    device_vk->vk_info = *vk_info;
+#define LOAD_DEVICE_PFN(name) \
+    if (!(device_vk->vk_info.vk_ops.name = (void *)VK_CALL(vkGetDeviceProcAddr(vk_device, #name)))) \
+    { \
+        WARN("Could not get device proc addr for '" #name "'.\n"); \
+        hr = E_FAIL; \
+        goto fail; \
+    }
+#define VK_DEVICE_PFN LOAD_DEVICE_PFN
+    VK_DEVICE_FUNCS()
+#undef VK_DEVICE_PFN
+
+    if (FAILED(hr = wined3d_device_init(&device_vk->d, wined3d, adapter->ordinal, device_type,
+            focus_window, flags, surface_alignment, levels, level_count, device_parent)))
+    {
+        WARN("Failed to initialize device, hr %#x.\n", hr);
+        goto fail;
+    }
+
+    *device = &device_vk->d;
+
+    return WINED3D_OK;
+
+fail:
+    VK_CALL(vkDestroyDevice(vk_device, NULL));
+    heap_free(device_vk);
+    return hr;
+}
+
+static void adapter_vk_destroy_device(struct wined3d_device *device)
+{
+    struct wined3d_device_vk *device_vk = wined3d_device_vk(device);
+    const struct wined3d_vk_info *vk_info = &device_vk->vk_info;
+
+    wined3d_device_cleanup(&device_vk->d);
+    VK_CALL(vkDestroyDevice(device_vk->vk_device, NULL));
+    heap_free(device_vk);
+}
+
+struct wined3d_context *adapter_vk_acquire_context(struct wined3d_device *device,
+        struct wined3d_texture *texture, unsigned int sub_resource_idx)
+{
+    TRACE("device %p, texture %p, sub_resource_idx %u.\n", device, texture, sub_resource_idx);
+
+    wined3d_from_cs(device->cs);
+
+    if (!device->context_count)
+        return NULL;
+
+    return &wined3d_device_vk(device)->context_vk;
+}
+
+void adapter_vk_release_context(struct wined3d_context *context)
+{
+    TRACE("context %p.\n", context);
 }
 
 static void adapter_vk_get_wined3d_caps(const struct wined3d_adapter *adapter, struct wined3d_caps *caps)
@@ -118,15 +324,16 @@ static void adapter_vk_get_wined3d_caps(const struct wined3d_adapter *adapter, s
     caps->Caps2 |= WINED3DCAPS2_CANGENMIPMAP;
 
     caps->PrimitiveMiscCaps |= WINED3DPMISCCAPS_BLENDOP
-            | WINED3DPMISCCAPS_SEPARATEALPHABLEND
             | WINED3DPMISCCAPS_INDEPENDENTWRITEMASKS
-            | WINED3DPMISCCAPS_POSTBLENDSRGBCONVERT;
+            | WINED3DPMISCCAPS_MRTINDEPENDENTBITDEPTHS
+            | WINED3DPMISCCAPS_POSTBLENDSRGBCONVERT
+            | WINED3DPMISCCAPS_SEPARATEALPHABLEND;
+
+    caps->RasterCaps |= WINED3DPRASTERCAPS_MIPMAPLODBIAS;
 
     if (sampler_anisotropy)
     {
-        caps->RasterCaps |= WINED3DPRASTERCAPS_ANISOTROPY
-                | WINED3DPRASTERCAPS_ZBIAS
-                | WINED3DPRASTERCAPS_MIPMAPLODBIAS;
+        caps->RasterCaps |= WINED3DPRASTERCAPS_ANISOTROPY;
 
         caps->TextureFilterCaps |= WINED3DPTFILTERCAPS_MAGFANISOTROPIC
                 | WINED3DPTFILTERCAPS_MINFANISOTROPIC;
@@ -207,12 +414,66 @@ static BOOL adapter_vk_check_format(const struct wined3d_adapter *adapter,
     return TRUE;
 }
 
+static HRESULT adapter_vk_init_3d(struct wined3d_device *device)
+{
+    struct wined3d_device_vk *device_vk;
+    struct wined3d_context *context_vk;
+    HRESULT hr;
+
+    TRACE("device %p.\n", device);
+
+    device_vk = wined3d_device_vk(device);
+    context_vk = &device_vk->context_vk;
+    if (FAILED(hr = wined3d_context_vk_init(context_vk, device->swapchains[0])))
+    {
+        WARN("Failed to initialise context.\n");
+        return hr;
+    }
+
+    if (!device_context_add(device, context_vk))
+    {
+        ERR("Failed to add the newly created context to the context list.\n");
+        wined3d_context_cleanup(context_vk);
+        return E_FAIL;
+    }
+
+    TRACE("Initialised context %p.\n", context_vk);
+
+    if (!(device_vk->d.blitter = wined3d_cpu_blitter_create()))
+    {
+        ERR("Failed to create CPU blitter.\n");
+        device_context_remove(device, context_vk);
+        wined3d_context_cleanup(context_vk);
+        return E_FAIL;
+    }
+
+    return WINED3D_OK;
+}
+
+static void adapter_vk_uninit_3d(struct wined3d_device *device)
+{
+    struct wined3d_context *context_vk;
+
+    TRACE("device %p.\n", device);
+
+    device->blitter->ops->blitter_destroy(device->blitter, NULL);
+
+    context_vk = &wined3d_device_vk(device)->context_vk;
+    device_context_remove(device, context_vk);
+    wined3d_context_cleanup(context_vk);
+}
+
 static const struct wined3d_adapter_ops wined3d_adapter_vk_ops =
 {
     adapter_vk_destroy,
-    adapter_vk_create_context,
+    adapter_vk_create_device,
+    adapter_vk_destroy_device,
+    adapter_vk_acquire_context,
+    adapter_vk_release_context,
     adapter_vk_get_wined3d_caps,
     adapter_vk_check_format,
+    adapter_vk_init_3d,
+    adapter_vk_uninit_3d,
 };
 
 static unsigned int wined3d_get_wine_vk_version(void)
@@ -222,7 +483,9 @@ static unsigned int wined3d_get_wine_vk_version(void)
 
     major = atoi(ptr);
 
-    while (isdigit(*ptr) || *ptr == '.')
+    while (isdigit(*ptr))
+        ++ptr;
+    if (*ptr == '.')
         ++ptr;
 
     minor = atoi(ptr);
@@ -261,7 +524,7 @@ static BOOL enable_vulkan_instance_extensions(uint32_t *extension_count,
 
     if ((vr = pfn_vkEnumerateInstanceExtensionProperties(NULL, &count, NULL)) < 0)
     {
-        WARN("Failed to count instance extensions, vr %d.\n", vr);
+        WARN("Failed to count instance extensions, vr %s.\n", wined3d_debug_vkresult(vr));
         goto done;
     }
     if (!(extensions = heap_calloc(count, sizeof(*extensions))))
@@ -271,7 +534,7 @@ static BOOL enable_vulkan_instance_extensions(uint32_t *extension_count,
     }
     if ((vr = pfn_vkEnumerateInstanceExtensionProperties(NULL, &count, extensions)) < 0)
     {
-        WARN("Failed to enumerate extensions, vr %d.\n", vr);
+        WARN("Failed to enumerate extensions, vr %s.\n", wined3d_debug_vkresult(vr));
         goto done;
     }
 
@@ -354,7 +617,7 @@ static BOOL wined3d_init_vulkan(struct wined3d_vk_info *vk_info)
 
     if ((vr = VK_CALL(vkCreateInstance(&instance_info, NULL, &instance))) < 0)
     {
-        WARN("Failed to create Vulkan instance, vr %d.\n", vr);
+        WARN("Failed to create Vulkan instance, vr %s.\n", wined3d_debug_vkresult(vr));
         goto fail;
     }
 
@@ -402,7 +665,7 @@ static VkPhysicalDevice get_vulkan_physical_device(struct wined3d_vk_info *vk_in
 
     if ((vr = VK_CALL(vkEnumeratePhysicalDevices(vk_info->instance, &count, NULL))) < 0)
     {
-        WARN("Failed to enumerate physical devices, vr %d.\n", vr);
+        WARN("Failed to enumerate physical devices, vr %s.\n", wined3d_debug_vkresult(vr));
         return VK_NULL_HANDLE;
     }
     if (!count)
@@ -419,36 +682,76 @@ static VkPhysicalDevice get_vulkan_physical_device(struct wined3d_vk_info *vk_in
 
     if ((vr = VK_CALL(vkEnumeratePhysicalDevices(vk_info->instance, &count, physical_devices))) < 0)
     {
-        WARN("Failed to get physical devices, vr %d.\n", vr);
+        WARN("Failed to get physical devices, vr %s.\n", wined3d_debug_vkresult(vr));
         return VK_NULL_HANDLE;
     }
 
     return physical_devices[0];
 }
 
-const struct wined3d_gpu_description *get_vulkan_gpu_description(const VkPhysicalDeviceProperties *properties)
+static enum wined3d_display_driver guess_display_driver(enum wined3d_pci_vendor vendor)
 {
-    const struct wined3d_gpu_description *description;
+    switch (vendor)
+    {
+        case HW_VENDOR_AMD:    return DRIVER_AMD_RX;
+        case HW_VENDOR_INTEL:  return DRIVER_INTEL_HD4000;
+        case HW_VENDOR_NVIDIA: return DRIVER_NVIDIA_GEFORCE8;
+        default:               return DRIVER_WINE;
+    }
+}
+
+static void adapter_vk_init_driver_info(struct wined3d_adapter *adapter,
+        const VkPhysicalDeviceProperties *properties, const VkPhysicalDeviceMemoryProperties *memory_properties)
+{
+    const struct wined3d_gpu_description *gpu_description;
+    struct wined3d_gpu_description description;
+    UINT64 vram_bytes, sysmem_bytes;
+    const VkMemoryHeap *heap;
+    unsigned int i;
 
     TRACE("Device name: %s.\n", debugstr_a(properties->deviceName));
     TRACE("Vendor ID: 0x%04x, Device ID: 0x%04x.\n", properties->vendorID, properties->deviceID);
     TRACE("Driver version: %#x.\n", properties->driverVersion);
     TRACE("API version: %s.\n", debug_vk_version(properties->apiVersion));
 
-    if ((description = wined3d_get_gpu_description(properties->vendorID, properties->deviceID)))
-        return description;
+    for (i = 0, vram_bytes = 0, sysmem_bytes = 0; i < memory_properties->memoryHeapCount; ++i)
+    {
+        heap = &memory_properties->memoryHeaps[i];
+        TRACE("Memory heap [%u]: flags %#x, size 0x%s.\n",
+                i, heap->flags, wine_dbgstr_longlong(heap->size));
+        if (heap->flags & VK_MEMORY_HEAP_DEVICE_LOCAL_BIT)
+            vram_bytes += heap->size;
+        else
+            sysmem_bytes += heap->size;
+    }
+    TRACE("Total device memory: 0x%s.\n", wine_dbgstr_longlong(vram_bytes));
+    TRACE("Total shared system memory: 0x%s.\n", wine_dbgstr_longlong(sysmem_bytes));
 
-    FIXME("Failed to retrieve GPU description for device %s %04x:%04x.\n",
-            debugstr_a(properties->deviceName), properties->vendorID, properties->deviceID);
+    if (!(gpu_description = wined3d_get_user_override_gpu_description(properties->vendorID, properties->deviceID)))
+        gpu_description = wined3d_get_gpu_description(properties->vendorID, properties->deviceID);
 
-    return wined3d_get_gpu_description(HW_VENDOR_AMD, CARD_AMD_RADEON_RX_VEGA);
+    if (!gpu_description)
+    {
+        FIXME("Failed to retrieve GPU description for device %s %04x:%04x.\n",
+                debugstr_a(properties->deviceName), properties->vendorID, properties->deviceID);
+
+        description.vendor = properties->vendorID;
+        description.device = properties->deviceID;
+        description.description = properties->deviceName;
+        description.driver = guess_display_driver(properties->vendorID);
+        description.vidmem = vram_bytes;
+
+        gpu_description = &description;
+    }
+
+    wined3d_driver_info_init(&adapter->driver_info, gpu_description, vram_bytes, sysmem_bytes);
 }
 
 static BOOL wined3d_adapter_vk_init(struct wined3d_adapter_vk *adapter_vk,
         unsigned int ordinal, unsigned int wined3d_creation_flags)
 {
     struct wined3d_vk_info *vk_info = &adapter_vk->vk_info;
-    const struct wined3d_gpu_description *gpu_description;
+    VkPhysicalDeviceMemoryProperties memory_properties;
     struct wined3d_adapter *adapter = &adapter_vk->a;
     VkPhysicalDeviceIDProperties id_properties;
     VkPhysicalDeviceProperties2 properties2;
@@ -456,7 +759,7 @@ static BOOL wined3d_adapter_vk_init(struct wined3d_adapter_vk *adapter_vk,
     TRACE("adapter_vk %p, ordinal %u, wined3d_creation_flags %#x.\n",
             adapter_vk, ordinal, wined3d_creation_flags);
 
-    if (!wined3d_adapter_init(adapter, ordinal))
+    if (!wined3d_adapter_init(adapter, ordinal, &wined3d_adapter_vk_ops))
         return FALSE;
 
     if (!wined3d_init_vulkan(vk_info))
@@ -479,12 +782,11 @@ static BOOL wined3d_adapter_vk_init(struct wined3d_adapter_vk *adapter_vk,
         VK_CALL(vkGetPhysicalDeviceProperties(adapter_vk->physical_device, &properties2.properties));
     adapter_vk->device_limits = properties2.properties.limits;
 
-    if (!(gpu_description = get_vulkan_gpu_description(&properties2.properties)))
-    {
-        ERR("Failed to get GPU description.\n");
-        goto fail_vulkan;
-    }
-    wined3d_driver_info_init(&adapter->driver_info, gpu_description, wined3d_settings.emulated_textureram);
+    VK_CALL(vkGetPhysicalDeviceMemoryProperties(adapter_vk->physical_device, &memory_properties));
+
+    adapter_vk_init_driver_info(adapter, &properties2.properties, &memory_properties);
+    adapter->vram_bytes_used = 0;
+    TRACE("Emulating 0x%s bytes of video ram.\n", wine_dbgstr_longlong(adapter->driver_info.vram_bytes));
 
     memcpy(&adapter->driver_uuid, id_properties.driverUUID, sizeof(adapter->driver_uuid));
     memcpy(&adapter->device_uuid, id_properties.deviceUUID, sizeof(adapter->device_uuid));
@@ -495,9 +797,9 @@ static BOOL wined3d_adapter_vk_init(struct wined3d_adapter_vk *adapter_vk,
     adapter->vertex_pipe = &none_vertex_pipe;
     adapter->fragment_pipe = &none_fragment_pipe;
     adapter->shader_backend = &none_shader_backend;
-    adapter->adapter_ops = &wined3d_adapter_vk_ops;
 
     adapter->d3d_info.wined3d_creation_flags = wined3d_creation_flags;
+    adapter->d3d_info.texture_swizzle = TRUE;
 
     return TRUE;
 
