@@ -67,7 +67,6 @@
 #undef CompareString
 #undef GetCurrentThread
 #undef _CDECL
-#undef DPRINTF
 #undef GetCurrentProcess
 #undef AnimatePalette
 #undef EqualRgn
@@ -103,7 +102,6 @@
 
 #undef ULONG
 #undef HRESULT
-#undef DPRINTF
 #undef STDMETHODCALLTYPE
 
 #include <assert.h>
@@ -204,48 +202,102 @@ static inline QTSplitter *impl_from_IBaseFilter( IBaseFilter *iface )
  * Base Filter
  */
 
-static IPin* WINAPI QT_GetPin(BaseFilter *iface, int pos)
+static IPin *qt_splitter_get_pin(BaseFilter *base, unsigned int index)
 {
-    QTSplitter *This = impl_from_BaseFilter(iface);
-    TRACE("Asking for pos %x\n", pos);
+    QTSplitter *filter = impl_from_BaseFilter(base);
 
-    if (pos > 2 || pos < 0)
-        return NULL;
-    switch (pos)
+    if (index == 0)
     {
-        case 0:
-            IPin_AddRef(&This->pInputPin.pin.IPin_iface);
-            return &This->pInputPin.pin.IPin_iface;
-        case 1:
-            if (This->pVideo_Pin)
-                IPin_AddRef(&This->pVideo_Pin->pin.pin.IPin_iface);
-            return &This->pVideo_Pin->pin.pin.IPin_iface;
-        case 2:
-            if (This->pAudio_Pin)
-                IPin_AddRef(&This->pAudio_Pin->pin.pin.IPin_iface);
-            return &This->pAudio_Pin->pin.pin.IPin_iface;
-        default:
-            return NULL;
+        IPin_AddRef(&filter->pInputPin.pin.IPin_iface);
+        return &filter->pInputPin.pin.IPin_iface;
     }
+    else if (index == 1)
+    {
+        if (filter->pVideo_Pin)
+        {
+            IPin_AddRef(&filter->pVideo_Pin->pin.pin.IPin_iface);
+            return &filter->pVideo_Pin->pin.pin.IPin_iface;
+        }
+        else if (filter->pAudio_Pin)
+        {
+            IPin_AddRef(&filter->pAudio_Pin->pin.pin.IPin_iface);
+            return &filter->pAudio_Pin->pin.pin.IPin_iface;
+        }
+    }
+    else if (index == 2 && filter->pVideo_Pin && filter->pAudio_Pin)
+    {
+        IPin_AddRef(&filter->pAudio_Pin->pin.pin.IPin_iface);
+        return &filter->pAudio_Pin->pin.pin.IPin_iface;
+    }
+
+    return NULL;
 }
 
-static LONG WINAPI QT_GetPinCount(BaseFilter *iface)
+static void qt_splitter_destroy(BaseFilter *iface)
 {
-    QTSplitter *This = impl_from_BaseFilter(iface);
-    int c = 1;
-    if (This->pAudio_Pin) c++;
-    if (This->pVideo_Pin) c++;
-    return c;
+    QTSplitter *filter = impl_from_BaseFilter(iface);
+    IPin *peer = NULL;
+    ULONG pinref;
+
+    EnterCriticalSection(&filter->csReceive);
+    /* Don't need to clean up output pins, disconnecting input pin will do that */
+    IPin_ConnectedTo(&filter->pInputPin.pin.IPin_iface, &peer);
+    if (peer)
+    {
+        IPin_Disconnect(peer);
+        IPin_Release(peer);
+    }
+    pinref = IPin_Release(&filter->pInputPin.pin.IPin_iface);
+    if (pinref)
+    {
+        ERR("pinref should be null, is %u, destroying anyway\n", pinref);
+        assert((LONG)pinref > 0);
+
+        while (pinref)
+            pinref = IPin_Release(&filter->pInputPin.pin.IPin_iface);
+    }
+
+    if (filter->pQTMovie)
+    {
+        DisposeMovie(filter->pQTMovie);
+        filter->pQTMovie = NULL;
+    }
+    if (filter->vContext)
+        QTVisualContextRelease(filter->vContext);
+    if (filter->aSession)
+        MovieAudioExtractionEnd(filter->aSession);
+
+    ExitMoviesOnThread();
+    LeaveCriticalSection(&filter->csReceive);
+
+    if (filter->loaderThread)
+    {
+        WaitForSingleObject(filter->loaderThread, INFINITE);
+        CloseHandle(filter->loaderThread);
+    }
+    if (filter->splitterThread)
+    {
+        SetEvent(filter->runEvent);
+        WaitForSingleObject(filter->splitterThread, INFINITE);
+        CloseHandle(filter->splitterThread);
+    }
+
+    CloseHandle(filter->runEvent);
+
+    filter->csReceive.DebugInfo->Spare[0] = 0;
+    DeleteCriticalSection(&filter->csReceive);
+    strmbase_filter_cleanup(&filter->filter);
+
+    CoTaskMemFree(filter);
 }
 
 static const BaseFilterFuncTable BaseFuncTable = {
-    QT_GetPin,
-    QT_GetPinCount
+    .filter_get_pin = qt_splitter_get_pin,
+    .filter_destroy = qt_splitter_destroy,
 };
 
-IUnknown * CALLBACK QTSplitter_create(IUnknown *punkout, HRESULT *phr)
+IUnknown * CALLBACK QTSplitter_create(IUnknown *outer, HRESULT *phr)
 {
-    IUnknown *obj = NULL;
     PIN_INFO *piInput;
     QTSplitter *This;
     static const WCHAR wcsInputPinName[] = {'I','n','p','u','t',' ','P','i','n',0};
@@ -255,7 +307,6 @@ IUnknown * CALLBACK QTSplitter_create(IUnknown *punkout, HRESULT *phr)
     RegisterWineDataHandler();
 
     This = CoTaskMemAlloc(sizeof(*This));
-    obj = (IUnknown*)This;
     if (!This)
     {
         *phr = E_OUTOFMEMORY;
@@ -263,7 +314,8 @@ IUnknown * CALLBACK QTSplitter_create(IUnknown *punkout, HRESULT *phr)
     }
     ZeroMemory(This,sizeof(*This));
 
-    BaseFilter_Init(&This->filter, &QT_Vtbl, &CLSID_QTSplitter, (DWORD_PTR)(__FILE__ ": QTSplitter.csFilter"), &BaseFuncTable);
+    strmbase_filter_init(&This->filter, &QT_Vtbl, outer, &CLSID_QTSplitter,
+            (DWORD_PTR)(__FILE__ ": QTSplitter.csFilter"), &BaseFuncTable);
 
     InitializeCriticalSection(&This->csReceive);
     This->csReceive.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__": QTSplitter.csReceive");
@@ -286,66 +338,7 @@ IUnknown * CALLBACK QTSplitter_create(IUnknown *punkout, HRESULT *phr)
     SourceSeeking_Init(&This->sourceSeeking, &QT_Seeking_Vtbl, QTSplitter_ChangeStop, QTSplitter_ChangeStart, QTSplitter_ChangeRate,  &This->filter.csFilter);
 
     *phr = S_OK;
-    return obj;
-}
-
-static void QT_Destroy(QTSplitter *This)
-{
-    IPin *connected = NULL;
-    ULONG pinref;
-
-    TRACE("Destroying\n");
-
-    EnterCriticalSection(&This->csReceive);
-    /* Don't need to clean up output pins, disconnecting input pin will do that */
-    IPin_ConnectedTo(&This->pInputPin.pin.IPin_iface, &connected);
-    if (connected)
-    {
-        IPin_Disconnect(connected);
-        IPin_Release(connected);
-    }
-    pinref = IPin_Release(&This->pInputPin.pin.IPin_iface);
-    if (pinref)
-    {
-        ERR("pinref should be null, is %u, destroying anyway\n", pinref);
-        assert((LONG)pinref > 0);
-
-        while (pinref)
-            pinref = IPin_Release(&This->pInputPin.pin.IPin_iface);
-    }
-
-    if (This->pQTMovie)
-    {
-        DisposeMovie(This->pQTMovie);
-        This->pQTMovie = NULL;
-    }
-    if (This->vContext)
-        QTVisualContextRelease(This->vContext);
-    if (This->aSession)
-        MovieAudioExtractionEnd(This->aSession);
-
-    ExitMoviesOnThread();
-    LeaveCriticalSection(&This->csReceive);
-
-    if (This->loaderThread)
-    {
-        WaitForSingleObject(This->loaderThread, INFINITE);
-        CloseHandle(This->loaderThread);
-    }
-    if (This->splitterThread)
-    {
-        SetEvent(This->runEvent);
-        WaitForSingleObject(This->splitterThread, INFINITE);
-        CloseHandle(This->splitterThread);
-    }
-
-    CloseHandle(This->runEvent);
-
-    This->csReceive.DebugInfo->Spare[0] = 0;
-    DeleteCriticalSection(&This->csReceive);
-    BaseFilter_Destroy(&This->filter);
-
-    CoTaskMemFree(This);
+    return &This->filter.IUnknown_inner;
 }
 
 static HRESULT WINAPI QT_QueryInterface(IBaseFilter *iface, REFIID riid, LPVOID *ppv)
@@ -379,19 +372,6 @@ static HRESULT WINAPI QT_QueryInterface(IBaseFilter *iface, REFIID riid, LPVOID
     return E_NOINTERFACE;
 }
 
-static ULONG WINAPI QT_Release(IBaseFilter *iface)
-{
-    QTSplitter *This = impl_from_IBaseFilter(iface);
-    ULONG refCount = InterlockedDecrement(&This->filter.refCount);
-
-    TRACE("(%p)->() Release from %d\n", This, refCount + 1);
-
-    if (!refCount)
-        QT_Destroy(This);
-
-    return refCount;
-}
-
 static HRESULT WINAPI QT_Stop(IBaseFilter *iface)
 {
     QTSplitter *This = impl_from_IBaseFilter(iface);
@@ -797,7 +777,7 @@ static HRESULT WINAPI QT_GetState(IBaseFilter *iface, DWORD dwMilliSecsTimeout,
 static const IBaseFilterVtbl QT_Vtbl = {
     QT_QueryInterface,
     BaseFilterImpl_AddRef,
-    QT_Release,
+    BaseFilterImpl_Release,
     BaseFilterImpl_GetClassID,
     QT_Stop,
     QT_Pause,
@@ -812,6 +792,25 @@ static const IBaseFilterVtbl QT_Vtbl = {
     BaseFilterImpl_QueryVendorInfo
 };
 
+static HRESULT break_source_connection(BaseOutputPin *pin)
+{
+    HRESULT hr;
+
+    EnterCriticalSection(pin->pin.pCritSec);
+    if (!pin->pin.pConnectedTo || !pin->pMemInputPin)
+        hr = VFW_E_NOT_CONNECTED;
+    else
+    {
+        hr = IMemAllocator_Decommit(pin->pAllocator);
+        if (SUCCEEDED(hr))
+            hr = IPin_Disconnect(pin->pin.pConnectedTo);
+        IPin_Disconnect(&pin->pin.IPin_iface);
+    }
+    LeaveCriticalSection(pin->pin.pCritSec);
+
+    return hr;
+}
+
 /*
  * Input Pin
  */
@@ -823,7 +822,7 @@ static HRESULT QT_RemoveOutputPins(QTSplitter *This)
     if (This->pVideo_Pin)
     {
         OutputQueue_Destroy(This->pVideo_Pin->queue);
-        hr = BaseOutputPinImpl_BreakConnect(&This->pVideo_Pin->pin);
+        hr = break_source_connection(&This->pVideo_Pin->pin);
         TRACE("Disconnect: %08x\n", hr);
         IPin_Release(&This->pVideo_Pin->pin.pin.IPin_iface);
         This->pVideo_Pin = NULL;
@@ -831,7 +830,7 @@ static HRESULT QT_RemoveOutputPins(QTSplitter *This)
     if (This->pAudio_Pin)
     {
         OutputQueue_Destroy(This->pAudio_Pin->queue);
-        hr = BaseOutputPinImpl_BreakConnect(&This->pAudio_Pin->pin);
+        hr = break_source_connection(&This->pAudio_Pin->pin);
         TRACE("Disconnect: %08x\n", hr);
         IPin_Release(&This->pAudio_Pin->pin.pin.IPin_iface);
         This->pAudio_Pin = NULL;
@@ -1428,25 +1427,6 @@ static HRESULT WINAPI QTOutPin_DecideAllocator(BaseOutputPin *iface, IMemInputPi
     return hr;
 }
 
-static HRESULT WINAPI QTOutPin_BreakConnect(BaseOutputPin *This)
-{
-    HRESULT hr;
-
-    TRACE("(%p)->()\n", This);
-
-    EnterCriticalSection(This->pin.pCritSec);
-    if (!This->pin.pConnectedTo || !This->pMemInputPin)
-        hr = VFW_E_NOT_CONNECTED;
-    else
-    {
-        hr = IPin_Disconnect(This->pin.pConnectedTo);
-        IPin_Disconnect(&This->pin.IPin_iface);
-    }
-    LeaveCriticalSection(This->pin.pCritSec);
-
-    return hr;
-}
-
 static const IPinVtbl QT_OutputPin_Vtbl = {
     QTOutPin_QueryInterface,
     BasePinImpl_AddRef,
@@ -1517,13 +1497,11 @@ static const IQualityControlVtbl QTOutPin_QualityControl_Vtbl = {
 static const BaseOutputPinFuncTable output_BaseOutputFuncTable = {
     {
         QTOutPin_CheckMediaType,
-        BaseOutputPinImpl_AttemptConnection,
-        BasePinImpl_GetMediaTypeVersion,
         QTOutPin_GetMediaType
     },
+    BaseOutputPinImpl_AttemptConnection,
     QTOutPin_DecideBufferSize,
     QTOutPin_DecideAllocator,
-    QTOutPin_BreakConnect
 };
 
 static const OutputQueueFuncTable output_OutputQueueFuncTable = {
