@@ -126,8 +126,11 @@ static void WINECON_RegLoadHelper(HKEY hConKey, struct config_data* cfg)
     count = sizeof(val);
     if (!RegQueryValueExW(hConKey, wszFontSize, 0, &type, (LPBYTE)&val, &count))
     {
-        cfg->cell_height = HIWORD(val);
-        cfg->cell_width  = LOWORD(val);
+        int height = HIWORD(val);
+        int width = LOWORD(val);
+        /* A value of zero reflects the default settings */
+        if (height != 0) cfg->cell_height = MulDiv( height, GetDpiForSystem(), USER_DEFAULT_SCREEN_DPI );
+        if (width != 0) cfg->cell_width = MulDiv( width, GetDpiForSystem(), USER_DEFAULT_SCREEN_DPI );
     }
 
     count = sizeof(val);
@@ -208,9 +211,9 @@ void WINECON_RegLoad(const WCHAR* appname, struct config_data* cfg)
     cfg->cursor_visible = 1;
     cfg->exit_on_die = 1;
     memset(cfg->face_name, 0, sizeof(cfg->face_name));
-    cfg->cell_height = 12;
-    cfg->cell_width  = 8;
-    cfg->font_weight = 0;
+    cfg->cell_height = MulDiv( 16, GetDpiForSystem(), USER_DEFAULT_SCREEN_DPI );
+    cfg->cell_width  = MulDiv( 8, GetDpiForSystem(), USER_DEFAULT_SCREEN_DPI );
+    cfg->font_weight = FW_NORMAL;
     cfg->history_size = 50;
     cfg->history_nodup = 0;
     cfg->insert_mode = 1;
@@ -256,7 +259,7 @@ void WINECON_RegLoad(const WCHAR* appname, struct config_data* cfg)
 static void WINECON_RegSaveHelper(HKEY hConKey, const struct config_data* cfg)
 {
     int   i;
-    DWORD val;
+    DWORD val, width, height;
     WCHAR color_name[13];
 
     WINECON_DumpConfig("save", cfg);
@@ -282,7 +285,9 @@ static void WINECON_RegSaveHelper(HKEY hConKey, const struct config_data* cfg)
 
     RegSetValueExW(hConKey, wszFaceName, 0, REG_SZ, (LPBYTE)&cfg->face_name, sizeof(cfg->face_name));
 
-    val = MAKELONG(cfg->cell_width, cfg->cell_height);
+    width  = MulDiv( cfg->cell_width, USER_DEFAULT_SCREEN_DPI, GetDpiForSystem() );
+    height = MulDiv( cfg->cell_height, USER_DEFAULT_SCREEN_DPI, GetDpiForSystem() );
+    val = MAKELONG(width, height);
     RegSetValueExW(hConKey, wszFontSize, 0, REG_DWORD, (LPBYTE)&val, sizeof(val));
 
     val = cfg->font_weight;
